@@ -5,6 +5,8 @@
 #define _ABOUT_DLG_H_
 #include "..\Manah\Dialog.h"
 #include "..\Manah\LinkLabel.h"
+using Manah::Windows::Controls::CDialog;
+using Manah::Windows::Controls::CLinkLabel;
 
 
 // CAboutDlg class definition
