@@ -1,26 +1,29 @@
 // Alpha.h
-// (c) 2003 exeal
+// (c) 2003-2004 exeal
 
 #ifndef _ALPHA_H_
 #define _ALPHA_H_
+#include "resource.h"
 #include "..\Manah\WinApp.h"
-#include "..\Manah\MenuOperator.h"
+#include "..\Manah\Menu.h"
 #include "AlphaTabCtrl.h"
 #include "AlphaEditController.h"
 #include "AlphaView.h"
 #include "Ambient.h"
 #include "OutputWnd.h"
-#include "FindDlg.h"
-#include "GotoLineDlg.h"
-#include "BookmarkDlg.h"
-#include "OutlineParserDlg.h"
+//#include "OutlineParserDlg.h"
 #include "KeyboardMap.h"
 #include "MRUManager.h"
-#include "PluginManager.h"
-#include "ScriptLanguageManager.h"
-#include "AlphaApplicationDebugger.h"
+#include "ScriptMacroManager.h"
+//#include "AlphaApplicationDebugger.h"
 #include <map>
 #include <sstream>
+using Manah::Text::CodePage;
+using Manah::Text::CPEX_AUTODETECT_USERLANG;
+using Manah::Windows::CWinApp;
+using Manah::Windows::CMessageArguments;
+using Manah::Windows::CImageList;
+using Manah::Windows::GDI::CMinimalDC;
 
 
 // Literals
@@ -29,62 +32,57 @@
 #define IDN_TOOLIMAGECOUNT	(32 * 7)
 
 #define IDS_UNTITLED			L"Untitled"
+#define IDS_DEFAULTSTATUSTEXT	L""
 #define IDS_FILEFILTERW			L"ׂẴt@C\0*.*\0\0"
 #define IDS_KEYMACROFILENAME	L"keymacro"
-
+#define IDS_EVENTSCRIPTFILENAME	L"events.*"
+#define IDS_MACRO_DIRECTORY_NAME			L"macros\\"
+#define IDS_KEYBOARDSCHEME_DIRECTORY_NAME	L"keyboardSchemes\\"
+#define IDS_ICON_DIRECTORY_NAME				L"icons\\"
+#define IDS_BREAK_CR	L"CR (Macintosh)"
+#define IDS_BREAK_LF	L"LF (Unix)"
+#define IDS_BREAK_CRLF	L"CR+LF (Windows)"
+#define IDS_BREAK_NEL	L"NEL (U+0085)"
+#define IDS_BREAK_LS	L"LS (U+2028)"
+#define IDS_BREAK_PS	L"PS (U+2029)"
 
 // Timer ID
 #define ID_TIMER_QUERYCOMMAND	1	// c[o[ACe̗L
 #define ID_TIMER_MOUSEMOVE		2	// J[\~1bɔ (qg\ȂǂɎgp)
 
+// message ID
+#define WM_EVENTHANDLER		WM_APP + 1	// CXbhCxgnhXNvgĂяo
+										// wParam => unused
+										// lParam => std::pair<const OLECHAR*, DISPPARAMS*>*
+										// lParam->first => Cxgnh
+										// lParam->second => 
+#define WM_CREATENEWTAB		WM_APP + 2	// OnFileNew() Ăяo
+										// wParam => 
+										// lParam => unused
+#define WM_ENDSCRIPTMACRO	WM_APP + 3	// }NXNvgIɑĂ
+										// wParam, lParam => unused
+
 
 namespace Alpha {
 
-static bool	arrAvailableIcon[32 * 7] = {	// ACRpӂĂ ID ɑ΂Đ^lz
-	true,	true,	false,	false,	true,	true,	true,	false,	// t@C
-	false,	false,	false,	false,	false,	false,	false,	false,
-	false,	false,	false,	false,	false,	false,	false,	false,
-	false,	false,	false,	false,	false,	false,	false,	false,
-	true,	true,	true,	true,	true,	true,	false,	false,	// ҏW
-	false,	false,	true,	true,	true,	true,	true,	true,
-	true,	true,	true,	true,	true,	true,	false,	false,
-	false,	false,	false,	false,	false,	false,	false,	false,
-	true,	true,	true,	false,	false,	false,	false,	true,	// 
-	false,	true,	true,	true,	false,	true,	false,	false,
-	false,	false,	false,	false,	false,	false,	false,	false,
-	false,	false,	false,	false,	false,	false,	false,	false,
-	false,	false,	true,	true,	true,	false,	true,	true,	// \
-	true,	false,	false,	false,	false,	false,	false,	false,
-	false,	false,	false,	false,	false,	false,	false,	false,
-	false,	false,	false,	false,	false,	false,	false,	false,
-	true,	false,	true,	true,	true,	true,	true,	true,	// fobO
-	true,	true,	true,	true,	true,	false,	false,	false,
-	false,	false,	false,	false,	false,	false,	false,	false,
-	false,	false,	false,	false,	false,	false,	false,	false,
-	false,	false,	false,	true,	true,	true,	true,	false,	// c[
-	true,	true,	false,	false,	false,	false,	false,	false,
-	false,	false,	false,	false,	false,	false,	false,	false,
-	false,	false,	false,	false,	false,	false,	false,	false,
-	false,	false,	false,	false,	false,	false,	false,	false,	// wv
-	false,	false,	false,	false,	false,	false,	false,	false,
-	false,	false,	false,	false,	false,	false,	false,	false,
-	false,	false,	false,	false,	false,	false,	false,	false,
-};
-
-
 // O錾
+class CEventHandlerScript;
+class CCommandManager;
+class CFindDlg;
 class CGotoLineDlg;
 class CBookmarkDlg;
-class CDebugBreakPointsDlg;
+/*class CDebugBreakPointsDlg;
 class CDebugCallStackDlg;
 class CDebugImmediateDlg;
 class CDebugRunningDocumentsDlg;
 class CDebugThreadsDlg;
-class CDebugVariablesDlg;
+class CDebugVariablesDlg;*/
 
+namespace Ambient {
+	class CApplication;
+	class CDocuments;
+}
 
-// 
-/////////////////////////////////////////////////////////////////////////////
 
 ///	s̑BCEditView::SetLineParam Ŏg
 enum LineAttribute {
@@ -98,20 +96,60 @@ enum LineAttribute {
 	LA_BREAKPOINT_MASK		= 0x03,
 };
 
+///	^Cvƃt@C̑Ή
+struct TDocumentType {
+	wstring	strName;				///	^Cv
+	wchar_t	wszFileSpec[MAX_PATH];	///	ChJ[hB MAX_PATH ȉłȂ΂ȂȂ
+	wstring	strCommand;				///	[s] ŎsR}hB"$F" ݂̃t@C\
+	bool	bPrivate;				///	Kp\XgB
+};
+
+typedef vector<TDocumentType>	DocTypeList;
+
+
+// CDocumentTypeManager class definition
+/////////////////////////////////////////////////////////////////////////////
+
+/**
+ *	@brief	^Cv̊Ǘ
+ *
+ *	ɊǗ镶^CvXgɂ̓CfbNXŃANZXB
+ *	0Ԗڂ̕^Cv͍ŏo^Ă镶^CvŁA
+ *	O͋󕶎Agq̃ftHg^CvłB
+ *	̕^Cv RemoveAll gĂ폜Ȃ
+ */
+class CDocumentTypeManager {
+	// RXgN^
+public:
+	CDocumentTypeManager();
+
+	// \bh
+public:
+	void					Add(const TDocumentType& type) throw(invalid_argument);
+	size_t					Find(const wstring& strName) const;
+	const TDocumentType&	GetAt(size_t iType) const throw(out_of_range);
+	const TDocumentType&	GetByFileName(const wstring& strFileName) const;
+	size_t					GetCount() const;
+	void					RemoveAll();
+
+	// f[^o
+private:
+	vector<TDocumentType>	m_vecDocTypes;
+};
+
 
 // CAlphaApp class definition
 /////////////////////////////////////////////////////////////////////////////
 
-struct TDocumentType;
-
 /// Alpha ̃AvP[VNX
 class CAlphaApp :
 		public CWinApp,
-		public IEditViewEventListener,
-		public IEditDocEventListener,
-		public IClipboardRingEventListener,
-		public IAlphaApplicationDebuggerEventListener {
+		public CEditViewEventAdapter,
+		virtual public IEditDocEventListener,
+		virtual public IClipboardRingEventListener/*,
+		virtual public IAlphaApplicationDebuggerEventListener*/ {
 	friend class CAlphaApplication;
+	friend class CCommandManager;
 
 	// f[^o
 protected:
@@ -120,10 +158,10 @@ protected:
 
 private:
 	/* EBhE */
-	CMenuOperator*		m_pMenu;			// Cj[
-	CPopupMenu*			m_pFileOpeMenu;		// [t@C] |bvAbvj[
-	CPopupMenu*			m_pNewDocTypeMenu;	// VKhLg^Cv|bvAbvj[
-	CPopupMenu*			m_pAppDocTypeMenu;	// KphLg^Cv|bvAbvj[
+	CMenu*				m_pMenu;			// Cj[
+	CMenu*				m_pFileOpeMenu;		// [t@C] |bvAbvj[
+	CMenu*				m_pNewDocTypeMenu;	// VKhLg^Cv|bvAbvj[
+	CMenu*				m_pAppDocTypeMenu;	// KphLg^Cv|bvAbvj[
 	HWND				m_hRebar;			// o[
 	HWND				m_hToolbar;			// c[o[
 	HWND				m_hDebugBar;		// fobOo[
@@ -133,53 +171,45 @@ private:
 	unsigned int		m_iLastActiveTab;	// OANeBu^u
 	COutputWnd			m_wndOutput;		// [o] EBhE
 	vector<IAlphaTab*>	m_vecDocuments;		// e^u
-	CFindDlg			m_dlgFind;			// []/[u]_CAO
+	CFindDlg*			m_pdlgSearch;		// [ƒu]_CAO
 	CGotoLineDlg*		m_pdlgGotoLine;		// [wsֈړ]_CAO
 	CBookmarkDlg*		m_pdlgBookmark;		// [ubN}[N]_CAO
-	COutlineParserDlg	m_dlgOutline;		// [AEgC]_CAO
-	CToolTipCtrl		m_oToolTip;			// fobÕc[qg
+//	COutlineParserDlg	m_dlgOutline;		// [AEgC]_CAO
 	WNDPROC				m_wndprocTab;		// ^ǔ̃EBhEvVW
 	int					m_nOutputWndHeight;	// o̓EBhE̍
 	bool				m_bSizingOutputWnd;	// o̓EBhE̍ύX
 
 	/* GDI IuWFNg */
+	LOGFONTW	m_lfCommonFont;
 	HFONT		m_hTabFont;						// ^uɎgptHg
 	CImageList	m_oTabIconsImageList;			// ^uɎgpC[WXg
-	HIMAGELIST	m_hCommandImageList;			// SR}h̃ACRi[C[WXg
-	HIMAGELIST	m_hDisabledCommandImageList;	// m_hCommandImageList ̃mNo[W
 
 	/* @\ƃR}h */
-	CAlphaApplicationDebugger*	m_pActiveDebugger;		// ANeBuȃfobK
+	CEventHandlerScript*		m_pEventHandlerScript;	// event.* s錾GW
+//	CAlphaApplicationDebugger*	m_pActiveDebugger;		// ANeBuȃfobK
 	CAlphaEditController*		m_pDebuggingEditor;		// fobÕGfB^
+	CCommandManager*			m_pCommandManager;		// R}h̊Ǘ
 	CKeyboardMap				m_oKeyboardMap;			// gp̃L[{[h}bv
 	CMRUManager*				m_pMRUManager;			// MRU Xg̊Ǘ
-	CScriptLanguageManager		m_oScriptLanguages;		// XNvg̊Ǘ
-	CPluginManager				m_oPluginManager;		// vOC̊Ǘ
+	CDocumentTypeManager		m_oDocTypeManager;		// ^Cv̊Ǘ
+	CScriptMacroManager*		m_pScriptMacroManager;	// XNvg}N̊Ǘ
 	bool						m_bCancelSelAfterCopy;	// Rs[sɑI邩
 
 	/* fobO_CAO */
-	CDebugBreakPointsDlg*		m_pdlgDebugBreakPoints;			// [u[N|Cg] _CAO
+/*	CDebugBreakPointsDlg*		m_pdlgDebugBreakPoints;			// [u[N|Cg] _CAO
 	CDebugCallStackDlg*			m_pdlgDebugCallStack;			// [Ăяo] _CAO
 	CDebugImmediateDlg*			m_pdlgDebugImmediate;			// [C~fBGCg] _CAO
 	CDebugRunningDocumentsDlg*	m_pdlgDebugRunningDocuments;	// [s̃hLg] _CAO
 	CDebugThreadsDlg*			m_pdlgDebugThreads;				// [Xbh] _CAO
 	CDebugVariablesDlg*			m_pdlgDebugVariables;			// [ϐ] _CAO
-
-	/* \Ȃ */
-	vector<TDocumentType>		m_vecDocumentTypes;	// ݓo^ĂhLg^Cv
-	vector<TDocumentStyleType>	m_vecDocStyleTypes;	// ݓo^ĂhLgX^C
-	LOGFONTW					m_lfCommonFont;		// r[ɎgptHg
-	unsigned int				m_nLeftMargin;		// r[Ɏgp鍶]
-	unsigned int				m_nTopMargin;		// r[Ɏgp]
-	MSXML2::IXMLDOMDocument*	m_pXMLDoc;			// PꃊXgp XML
-
+*/
 	/* I[g[VpC^[tFCX */
-	IApplication*	m_pApplication;
-	IDocuments*		m_pDocuments;
+	Alpha::Ambient::CApplication*	m_pApplication;
+	Alpha::Ambient::CDocuments*		m_pDocuments;
 
 	/* IvV */
 	bool	m_bShowMessageBoxOnFind;			// ŃbZ[W{bNX\
-	bool	m_bInitializeFindTextFromEditor;	// []/[u] _CAOJƂ
+	bool	m_bInitializeFindTextFromEditor;	// [ƒu] _CAOJƂ
 												// eLXgGfB^珉
 
 protected:
@@ -193,45 +223,42 @@ public:
 
 	// \bh
 public:
-	void					ApplyDocumentType(unsigned int iTab, unsigned int iDocType = -1) throw(out_of_range);
-	int						ErrorMessageBox(unsigned int nErrorId, unsigned int nType);
-	CAlphaDoc*				GetActiveDocument() const;
+	void					ApplyDocumentType(unsigned int iTab, const wstring& strTypeName = L"") throw(out_of_range);
 	IAlphaTab*				GetActiveTab() const;
 	unsigned int			GetActiveTabIndex() const;
-	CAlphaEditController*	GetActiveTextEditor() const;
-	void					GetAutomation(IApplication** ppApplication);
-	void					GetAutomationDocuments(IDocuments** ppDocuments);
-	CAlphaEditController*	GetTextEditor(unsigned int iTab) const throw(out_of_range);
-	unsigned int			GetDocumentTypeFromFileName(const wstring& strFileName) const;
+	HRESULT					GetAutomation(
+								Alpha::Ambient::CApplication** ppApplication,
+								Alpha::Ambient::CDocuments** ppDocuments) const;
+	static const wchar_t*	GetCodePageName(CodePage cp);
+	const CCommandManager&	GetCommandManager() const;
+	CDocumentTypeManager*	GetDocumentTypeManager() const;
+	CKeyboardMap&			GetKeyboardMap() const;
 	static CAlphaApp*		GetInstance();
 	IAlphaTab*				GetTab(unsigned int iTab) const throw(out_of_range);
 	unsigned int			GetTabCount() const;
 	virtual bool			InitApplication(HINSTANCE hInstance);
 	virtual bool			InitInstance();
-	wstring					LoadErrorString(unsigned int nErrorId) const;
-	int						MessageBox(DWORD wID, UINT nType);
-	int						MessageBox(DWORD wID, UINT nType, const CMessageArguments& args);
-	bool					OpenFile(const wstring& strPathName,
-								UINT nCodePage = EXTCP_AUTO, unsigned int DocType = -1);
+	int						MessageBox(DWORD wID, UINT nType, CMessageArguments& args = CMessageArguments());
+	bool					OpenFile(const wstring& strPathName, CodePage cp = CPEX_AUTODETECT_USERLANG,
+								const wstring& strDocTypeName = L"", bool bAddToMRU = true);
 	void					ParseCommandLine(const vector<wstring>& vecArgs);
 	void					SetActiveTab(unsigned int iTab) throw(out_of_range);
 
 	// ZbgAbv (INI t@CȂǂݒǂݍ)
 	void	LoadINISettings();
-	void	SetupKeyBinds(const wstring& strSchemeName);
+	void	LoadKeyBinds(const wstring& strSchemeName);
 	void	SetupMenus();
-	void	SetupTextFoundations();
 	void	SetupToolbar();
 
 	// IEditViewEventListener C^[tFCX
-	void	OnChangeLines(unsigned long iLine, unsigned long cLines);
-	void	OnCreateLines(unsigned long iLine, unsigned long cLines);
-	void	OnDeleteLines(unsigned long iLine, unsigned long cLines);
-	void	OnDrawIndicatorMargin(unsigned long iLine, CMinimalDC& oDC,
+	void	OnChangedAbbreviationExpansionReadyState(bool bReady, const string_t& strAbbrev);
+	void	OnDrawIndicatorMargin(Ascension::length_t iLine, CMinimalDC& oDC,
 				const RECT& rectMargin, bool bBookmarked, DWORD dwParam, bool bSingleEmphasis);
+	void	OnIncrementalSearchEvent(IncrementalSearchEvent event, const char_t* pwsz);
+	void	OnInvokeUriLink(const char_t* pwszUri);
 	void	OnMoveCaret(const CCharPos& pos);
 	void	OnMoveCursor(const POINT& pt);
-	void	OnQueryLineColor(unsigned long iLine, bool bBookmarked, DWORD dwParam,
+	void	OnQueryLineColor(Ascension::length_t iLine, bool bBookmarked, DWORD dwParam,
 				bool bSingleEmphasis, COLORREF& clrFore, COLORREF& clrBack);
 
 	// IEditDocEventListener C^[tFCX
@@ -241,7 +268,7 @@ public:
 	// IClipboardRingEventListener C^[tFCX
 	void	OnClipboardRingChanged();
 	void	OnClipboardRingDeniedAdding();
-
+/*
 	// IAlphaApplicationDebuggerEventListener C^[tFCX
 	void	OnDebugStart();
 	void	OnDebugEnd();
@@ -249,9 +276,8 @@ public:
 	void	OnDebugStopByError(unsigned long iLine);
 	void	OnDebugStepTo(unsigned long iLine);
 	void	OnDebugIllegalStatus(DebugStatus ds);
-
+*/
 protected:
-	void			BindOneKey(short sVKey, KeyModifier km, int nCommandID);
 	virtual LRESULT	DispatchEvent(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
 	void			GetModelessDialogList(list<HWND>& listDlgs) const;
 	bool			HandleFileOpenError(Ascension::StreamStatus ss);
@@ -262,7 +288,7 @@ protected:
 	// bZ[Wnh
 protected:
 	virtual bool	OnFileClose(bool bConfirm = true);
-	virtual bool	OnFileCloseAll();
+	virtual bool	OnFileCloseAll(bool bExceptCurrent);
 	virtual void	OnFileNew(UINT nCodePage);
 	virtual void	OnFileNewWithFormat();
 	virtual bool	OnFileOpen();
@@ -278,7 +304,6 @@ protected:
 	virtual void	OnFindFindNext(bool bPrevious = false, bool bNotice = true);
 	virtual void	OnFindBookmarkAll();
 	virtual void	OnFindToggleBookmark();
-	virtual void	OnFindReplace();
 	virtual void	OnFindReplaceAll();
 	virtual void	OnFindReplaceAndNext();
 	virtual void	OnFindParseOutline();
@@ -305,7 +330,10 @@ protected:
 	virtual void	OnDropFiles(HDROP hDrop);									// WM_DROPFILES
 	virtual void	OnEnterMenuLoop(bool bIsTrackPopup);						// WM_ENTERMENULOOP
 	virtual void	OnExitMenuLoop(bool bIsTrackPopup);							// WM_EXITMENULOOP
+	virtual void	OnInitMenu(HMENU hMenu);									// WM_INITMENU
+	virtual void	OnInitMenuPopup(HMENU hMenu, UINT nIndex, bool bSysMenu);	// WM_INITMENUPOPUP
 	virtual void	OnMeasureItem(UINT idCtrl, LPMEASUREITEMSTRUCT lpMI);		// WM_MEASUREITEM
+	virtual LRESULT	OnMenuChar(wchar_t ch, UINT nFlags, CMenu* pMenu);			// WM_MENUCHAR
 	virtual void	OnMenuSelect(UINT nItemID, UINT nFlags, HMENU hSysMenu);	// WM_MENUSELECT
 	virtual void	OnMouseMove(UINT nFlags, POINT pt);							// WM_MOUSEMOVE
 	virtual bool	OnNotify(int idCtrl, LPNMHDR lpNMHDR);						// WM_NOTIFY
@@ -313,6 +341,9 @@ protected:
 	virtual void	OnSize(UINT nType, int cx, int cy);							// WM_SIZE
 	virtual void	OnTimer(UINT nIDTimer);										// WM_TIMER
 
+protected:
+	void	OnRebarChevronPushed(LPNMREBARCHEVRON lpNMRebarChevron);	// RBN_CHEVRONPUSHED
+
 	// hLgAr[̕ύXĎ
 protected:
 	static void OnUpdateDocument();
@@ -323,33 +354,35 @@ protected:
 
 	// J[eBeB
 protected:
-	wchar_t*	GetMenuLabel(int nCommand) const;
-	void		GetProfileList(const wchar_t* pwszSectionName,
-					const wchar_t* pwszKeyName, list<wstring>& listItems, const wchar_t* pwszDefault = 0);
-	void		GetProfileSet(const wchar_t* pwszSectionName,
-					const wchar_t* pwszKeyName, set<wstring>& setItems, const wchar_t* pwszDefault = 0);
+	const wchar_t*	GetMenuLabel(CommandId id) const;
+	void			GetProfileList(const wchar_t* pwszSectionName,
+						const wchar_t* pwszKeyName, list<wstring>& listItems, const wchar_t* pwszDefault = 0);
+	void			GetProfileSet(const wchar_t* pwszSectionName,
+						const wchar_t* pwszKeyName, set<wstring>& setItems, const wchar_t* pwszDefault = 0);
 };
 
+
+///	R}hǗIuWFNgԂ
+inline const CCommandManager& CAlphaApp::GetCommandManager() const {
+	AssertValid();
+	return *m_pCommandManager;
+}
+
+///	^Cv}l[WA
+inline CDocumentTypeManager* CAlphaApp::GetDocumentTypeManager() const {
+	AssertValid();
+	return &const_cast<CAlphaApp*>(this)->m_oDocTypeManager;
+}
+
 ///	B̃AvP[VIuWFNgԂ
 inline CAlphaApp* CAlphaApp::GetInstance() {
 	return CAlphaApp::m_pInstance;
 }
 
-/**
- *	R}h̃j[LvVԂ
- *	@param nCommand	R}h ID
- */
-inline wchar_t*	CAlphaApp::GetMenuLabel(int nCommand) const {
-	static wchar_t	wsz[100];
-	wchar_t*		psz = 0;
-
-	LoadString(nCommand, wsz, 100);
-	psz = wcschr(wsz, L'\n');
-	if(psz != 0)
-		*psz = 0;
-	wcscat(wsz, L"\t");
-	wcscat(wsz, m_oKeyboardMap.GetKeyString(nCommand).c_str());
-	return wsz;
+///	L[蓖ăIuWFNgԂ
+inline CKeyboardMap& CAlphaApp::GetKeyboardMap() const {
+	AssertValid();
+	return const_cast<CAlphaApp*>(this)->m_oKeyboardMap;
 }
 
 /**
