@@ -1,5 +1,5 @@
 // AlphaDoc.cpp
-// (c) 2003 exeal
+// (c) 2003-2004 exeal
 
 #include "StdAfx.h"
 #include "AlphaDoc.h"
@@ -14,7 +14,7 @@ using Alpha::CAlphaView;
 
 ///	RXgN^
 CAlphaDoc::CAlphaDoc(Alpha::CAlphaEditController* pController /* = 0 */)
-		: CEditDoc(pController), m_pAutomation(0), m_iDocType(0) {
+		: CEditDoc(pController), m_pAutomation(0) {
 }
 
 ///	fXgN^
@@ -24,12 +24,12 @@ CAlphaDoc::~CAlphaDoc() {
 }
 
 ///	I[g[VpC^[tFCXIuWFNgԂ
-void CAlphaDoc::GetAutomation(Alpha::CAlphaApp* pApp, IDocument** ppDocument) {
+void CAlphaDoc::GetAutomation(Alpha::CAlphaApp& app, IDocument** ppDocument) {
 	AssertValid();
 	assert(ppDocument != 0);
 
 	if(m_pAutomation == 0) {	// ܂΍쐬
-		m_pAutomation = new Alpha::Ambient::CTextDocument(pApp, this);
+		m_pAutomation = new Alpha::Ambient::CTextDocument(app, *this);
 		m_pAutomation->AddRef();
 	}
 	*ppDocument = m_pAutomation;
