@@ -45,13 +45,6 @@ public:
 /////////////////////////////////////////////////////////////////////////////
 
 class CAlphaDoc : public Ascension::CEditDoc {
-	// f[^o
-protected:
-	IDocument*	m_pAutomation;	// I[g[VpC^[tFCX
-public:
-	unsigned int	m_iDocType;	// KpĂhLg^CvBw̖Ƃ0
-
-
 	// RXgN^
 public:
 	CAlphaDoc(CAlphaEditController* pController = 0);
@@ -59,8 +52,14 @@ public:
 
 	// \bh
 public:
-	void					GetAutomation(CAlphaApp* pApp, IDocument** ppDocument); // I[g[VIuWFNgԂ
+	void					GetAutomation(CAlphaApp& app, IDocument** ppDocument);
 	CAlphaEditController*	GetController() const;
+
+	// f[^o
+protected:
+	IDocument*	m_pAutomation;	// I[g[VpC^[tFCX
+public:
+	wstring	m_strDocType;	// KpĂ镶^Cv
 };
 
 
