@@ -11,6 +11,9 @@
 // CAlphaEditController class definition
 /////////////////////////////////////////////////////////////////////////////
 
+interface ITextProcessor;
+interface ITextSelection;
+
 namespace Alpha {
 
 class CAlphaDoc;
@@ -18,40 +21,38 @@ class CAlphaView;
 
 class CAlphaEditController :
 		public Ascension::CEditController,
-		public IAlphaTab {
-	// ^
-public:
-	typedef CAlphaDoc	DocumentClass;
-	typedef CAlphaView	ViewClass;
-
+		virtual public IAlphaTab {
 	// RXgN^
 public:
 	CAlphaEditController();
+	~CAlphaEditController();
 
 	// \bh
 public:
-	bool		Create(HWND hwndParent, const RECT* lpRect,
-						DWORD dwStyle, DWORD dwExStyle,
-						DWORD dwViewStyle, DWORD dwViewExStyle);
 	CAlphaView*	GetActiveView() const;
+	HRESULT		GetAutomation(REFIID riid, IDispatch** ppAutomation) const;
 	CAlphaDoc*	GetDocument() const;
 	CAlphaView*	GetPrimaryView() const;
+protected:
+	virtual Manah::Windows::CDocument*	_NewDocumentInstance();
+	virtual Manah::Windows::CView*		_NewViewInstance();
+	virtual Ascension::CEditView*		_NewViewInstance(const Ascension::CEditView& rhs);
 
 	// IAlphaTab C^[tFCX
 public:
-	void	DeleteInstance();
-	wstring	GetDescription() const;
-	wstring	GetTabTitle() const;
-	HWND	GetWindow() const;
-	bool	IsTextEditor() const;
-	void	RefreshWindow();
-	void	OnActivate(bool bActive);
-	bool	OnClosing();
-	void	OnSize(const RECT* lpRect);
+	void					DeleteInstance();
+	wstring					GetDescription() const;
+	wstring					GetFilePath() const;
+	wstring					GetTabTitle() const;
+	HWND					GetWindow() const;
+	CAlphaEditController*	GetTextEditor();
+	void					RefreshWindow();
+	void					OnActivateTab(bool bActive);
 
 	// f[^o
 private:
-	unsigned short	m_iLastActiveView;	// ŌɃtH[JXĂr[̔ԍ
+	mutable ITextProcessor*	m_pAutomationProcessor;
+	mutable ITextSelection*	m_pAutomationSelection;
 };
 
 
@@ -72,6 +73,12 @@ inline CAlphaDoc* CAlphaEditController::GetDocument() const {
 	return reinterpret_cast<CAlphaDoc*>(m_pDocument);
 }
 
+///	@see	IAlphaTab::GetFilePath
+inline wstring CAlphaEditController::GetFilePath() const {
+	AssertValid();
+	return m_pDocument->GetPathName();
+}
+
 ///	@see	CController::GetPrimaryView
 inline CAlphaView* CAlphaEditController::GetPrimaryView() const {
 	AssertValid();
@@ -80,6 +87,18 @@ inline CAlphaView* CAlphaEditController::GetPrimaryView() const {
 	return reinterpret_cast<CAlphaView*>(Ascension::CEditController::GetPrimaryView());
 }
 
+///	@see	IAlpha::GetTextEditor
+inline CAlphaEditController* CAlphaEditController::GetTextEditor() {
+	AssertValid();
+	return this;
+}
+
+///	@see	IAlphaTab::GetWindow
+inline HWND CAlphaEditController::GetWindow() const {
+	AssertValid();
+	return m_hWnd;
+}
+
 } // namespace Alpha
 
 
