@@ -1,5 +1,5 @@
 // AlphaInterfaces.h
-// (c) 2003 exeal
+// (c) 2003-2004 exeal
 
 #ifndef _ALPHA_INTERFACES_H_
 #define _ALPHA_INTERFACES_H_
@@ -7,6 +7,9 @@
 
 namespace Alpha {
 
+///	R}h̎ʎq ()
+typedef WORD	CommandId;	// typeof(ACCEL::cmd)
+
 // CAlphaApp ɂC^[tFCXA
 /////////////////////////////////////////////////////////////////////////////
 
