@@ -4,14 +4,15 @@
 #include "StdAfx.h"
 #include "EditDoc.h"
 #include <MAPI.h>	// MAPISendMail
-#include "..\..\Manah\Text.h"
 #include "..\..\Manah\WaitCursor.h"
 #include "..\..\Armaiti\ComGeneric.h"
+#include <algorithm>	// std::for_each
 
 using namespace Ascension;
 using Ascension::CEditDoc::CUndoManager;
 using namespace Manah;
 using namespace Manah::Windows;
+using namespace Manah::Windows::IO;
 using namespace Manah::Text;
 using namespace Armaiti;
 using namespace std;
@@ -22,17 +23,20 @@ using namespace std;
 
 ///	RXgN^
 CInsertOperation::CInsertOperation(const CCharPos& pos,
-		const wstring& strText) : m_bType(true), m_pos(pos), m_strText(strText) {
+		const string_t& strText) : m_bType(true), m_pos(pos), m_strText(strText) {
 }
 
 ///	폜͏ɌłȂ
 void CInsertOperation::Concat(COperationUnit* pOperationUnit) {
+	AssertValid();
 	assert(pOperationUnit != 0);
 	pOperationUnit->Push(this);
 }
 
 ///	}̍Đ
 void CInsertOperation::Execute(CEditDoc* pDoc) {
+	AssertValid();
+
 	VIEW_POSITION	pos = pDoc->GetFirstViewPosition();
 	CView*			pView;
 
@@ -54,6 +58,7 @@ CDeleteOperation::CDeleteOperation(const CCharPos& posBegin,
 
 ///	̑w肳ꂽPʂɌ
 void CDeleteOperation::Concat(COperationUnit* pOperationUnit) {
+	AssertValid();
 	assert(pOperationUnit != 0);
 	IOperation*	pLastOpe = pOperationUnit->Top();
 
@@ -70,6 +75,8 @@ void CDeleteOperation::Concat(COperationUnit* pOperationUnit) {
 
 ///	폜̍Đ
 void CDeleteOperation::Execute(CEditDoc* pDoc) {
+	AssertValid();
+
 	VIEW_POSITION	pos = pDoc->GetFirstViewPosition();
 	CView*			pView;
 
@@ -97,6 +104,7 @@ COperationUnit::~COperationUnit() {
  *	@param pDoc	ΏۃhLg
  */
 void COperationUnit::Execute(CEditDoc* pDoc) {
+	AssertValid();
 	while(!m_stkOpes.empty()) {	// SĂ̑s
 		m_stkOpes.top()->Execute(pDoc);
 		delete m_stkOpes.top();
@@ -106,6 +114,7 @@ void COperationUnit::Execute(CEditDoc* pDoc) {
 
 ///	1̑|bv
 IOperation* COperationUnit::Pop() {
+	AssertValid();
 	IOperation*	p = m_stkOpes.top();
 	m_stkOpes.pop();
 	return p;
@@ -113,6 +122,7 @@ IOperation* COperationUnit::Pop() {
 
 ///	1̑vbV
 void COperationUnit::Push(IOperation* pOperation) {
+	AssertValid();
 	if(m_stkOpes.size() != 0
 			&& !static_cast<CDeleteOperation*>(pOperation)->m_bType
 			&& !static_cast<CDeleteOperation*>(m_stkOpes.top())->m_bType) {
@@ -130,6 +140,7 @@ void COperationUnit::Push(IOperation* pOperation) {
 
 ///	擪̑擾
 IOperation* COperationUnit::Top() const {
+	AssertValid();
 	return m_stkOpes.top();
 }
 
@@ -139,7 +150,7 @@ IOperation* COperationUnit::Top() const {
 
 ///	RXgN^
 CUndoManager::CUndoManager(CEditDoc* pDoc)
-	: m_pDoc(pDoc), m_bVirtual(false), m_pVirtualUnit(0), m_pLastUnit(0) {
+	: m_pDoc(pDoc), m_bVirtual(false), m_pVirtualUnit(0), m_pLastUnit(0), m_pSavedOperation(0) {
 }
 
 ///	fXgN^
@@ -150,6 +161,7 @@ CUndoManager::~CUndoManager() {
 ///	AhD/hDX^bNɂ
 void CUndoManager::Clear() {
 	m_pLastUnit = 0;
+	m_pSavedOperation = 0;
 	while(!m_stkUndo.empty()) {
 		delete m_stkUndo.top();
 		m_stkUndo.pop();
@@ -170,6 +182,18 @@ size_t CUndoManager::GetUndoBufferLength() const {
 	return m_stkUndo.size();
 }
 
+///	Oۑ瑀삪ǉA폜ꂽԂ
+bool CUndoManager::IsModifiedSinceLastSave() const {
+	if(m_stkUndo.empty())
+		return m_pSavedOperation != 0;
+	return m_pSavedOperation != m_stkUndo.top()->Top();
+}
+
+///	ۑꂽƂʒm
+void CUndoManager::OnSave() {
+	m_pSavedOperation = !m_stkUndo.empty() ? m_stkUndo.top()->Top() : 0;
+}
+
 /**
  *	1̑AhDX^bNɒǉ
  *	@param pOperation	ǉ鑀
@@ -233,7 +257,7 @@ void CUndoManager::Undo() {
 // CEditDoc class implementation
 /////////////////////////////////////////////////////////////////////////////
 
-const wchar_t CEditDoc::m_wszBreakChars[6] = {		// U+0085 Ƀeg VC6 ŃG[ɂȂ
+const char_t CEditDoc::m_wszBreakChars[6] = {		// U+0085 Ƀeg VC6 ŃG[ɂȂ
 	0x000A, 0x000D, 0x0085, 0x2028, 0x2029, 0x0000	// (C99 ł 0x00A0 ȉ̕ \uxxxx `Ŏw肷邱Ƃ
 };													// oȂ炵 C++ Ƃ̊֘A͕s)
 
@@ -241,13 +265,11 @@ map<UINT, CEditDoc*> CEditDoc::m_mapDocuments;
 
 ///	RXgN^
 CEditDoc::CEditDoc(Ascension::CEditController* pController /* = 0 */) :
-		Manah::Windows::CDocument((CController*)pController), m_bIgnoreViews(false), m_bReadOnly(false),
-		m_fomMode(FOM_DENYNONE), m_breakType(BT_CRLF), m_ugsGroupingState(UGS_NONE),
-		m_bOnceUndoBufferCleared(false), m_bVirtualOperating(false), m_pEventListener(0),
+		CDocument((CController*)pController), m_bIgnoreViews(false), m_bReadOnly(false),
+			m_fomMode(FOM_DENYNONE), m_nCodePage(::GetACP()), m_breakType(BT_CRLF), m_ugsGroupingState(UGS_NONE),
+		m_bOnceUndoBufferCleared(false), m_bVirtualOperating(false),
 		m_itCache_(m_listLines.end()), m_itCache(m_listLines.end()) {
-	CEditDocLine	oLine;
-	m_nCodePage = ::GetACP();
-	m_listLines.push_back(oLine);
+	m_listLines.push_back(CEditDocLine());
 	m_pUndoManager = new CUndoManager(this);
 	m_nTimerId = ::SetTimer(0, 0, 1000, CEditDoc::TimerProc);
 	CEditDoc::m_mapDocuments[m_nTimerId] = this;
@@ -258,58 +280,39 @@ CEditDoc::~CEditDoc() {
 	CloseDocument();
 	delete m_pUndoManager;
 	::KillTimer(0, m_nTimerId);
-	map<UINT, CEditDoc*>::iterator	it = CEditDoc::m_mapDocuments.find(m_nTimerId);
-	if(it != CEditDoc::m_mapDocuments.end())
-		CEditDoc::m_mapDocuments.erase(it);
-}
-
-///	AhDO[v̎WJnB
-///	ݎWł΁AxIAVWJn
-void CEditDoc::BeginEditCollection() {
-	AssertValid();
-
-	if(m_ugsGroupingState != UGS_NONE)
-		EndEditCollection();
-	m_ugsGroupingState = UGS_WAITFORFIRSTEDIT;
+	CEditDoc::m_mapDocuments.erase(m_nTimerId);
 }
 
 ///	t@C̍ŏIXV`FbN
 void CEditDoc::CheckFileLastWriteTime() {
 	AssertValid();
 
-	if(m_strPathName.empty())
+	static bool	bChecking = false;
+
+	if(m_strPathName.empty() || bChecking)
 		return;
+	bChecking = true;
 
 	HANDLE				hFind;
 	WIN32_FIND_DATAW	wfd;
 
 	hFind = ::FindFirstFileW(m_strPathName.c_str(), &wfd);
-	if(hFind == INVALID_HANDLE_VALUE)
+	if(hFind == INVALID_HANDLE_VALUE) {
 		return;
-	if(::CompareFileTime(&m_lastWriteTime, &wfd.ftLastWriteTime) == -1
-			&& m_pEventListener != 0) {
+		bChecking = false;
+	}
+	if(::CompareFileTime(&m_lastWriteTime, &wfd.ftLastWriteTime) == -1) {
+		m_lastWriteTime = wfd.ftLastWriteTime;
+		for(set<IEditDocEventListener*>::iterator it
+				= m_setEventListeners.begin(); it != m_setEventListeners.end(); ++it)
+			(*it)->OnDocumentOverwrittenByOtherProcess(this);
+		// ʒmɃt@CXV邩Ȃ̂ŁA1xŐṼf[^
+		::FindClose(hFind);
+		hFind = ::FindFirstFileW(m_strPathName.c_str(), &wfd);
 		m_lastWriteTime = wfd.ftLastWriteTime;
-		m_pEventListener->OnDocumentOverwrittenByOtherProcess(this);
 	}
 	::FindClose(hFind);
-}
-
-///	AhD/hDX^bNɂė
-void CEditDoc::ClearUndoBuffer() {
-	AssertValid();
-	m_pUndoManager->Clear();
-	m_bOnceUndoBufferCleared = true;
-}
-
-///	hLg OnCloseDocument Ă
-bool CEditDoc::CloseDocument() {
-	AssertValid();
-
-	m_oFile.Close();
-	InitiateDocument();
-
-	OnCloseDocument();
-	return true;
+	bChecking = false;
 }
 
 /**
@@ -321,19 +324,19 @@ OperationStatus CEditDoc::CopyCurrentFile(const wchar_t* pwszDestination) {
 	AssertValid();
 	assert(pwszDestination != 0);
 
-	OperationStatus	ops = OPS_OK;
-	UINT			nShareMode;
-
 	if(m_strPathName.empty())
 		return OPS_HASNOINSTANCE;
-	if(::PathFileExistsW(pwszDestination))
+	if(toBoolean(::PathFileExistsW(pwszDestination)))
 		return OPS_ALREADYEXISTS;
 
+	OperationStatus	ops = OPS_OK;
+	UINT			nReopenMode = CFile::modeNoTruncate;
+
 	switch(static_cast<int>(m_fomMode)) {
-		case FOM_DENYWRITE:	nShareMode = CFile::shareDenyWrite;	break;
-		case FOM_DENYREAD:	nShareMode = CFile::shareDenyRead;	break;
+		case FOM_DENYWRITE:		nReopenMode |= CFile::modeRead | CFile::shareDenyWrite;		break;
+		case FOM_DENYREAD:		nReopenMode |= CFile::modeReadWrite | CFile::shareDenyRead;	break;
 		case FOM_DENYNONE:
-		case FOM_ASREADONLY:nShareMode = CFile::shareDenyNone;	break;
+		case FOM_ASREADONLY:	nReopenMode |= CFile::modeRead | CFile::shareDenyNone;		break;
 	}
 
 	m_oFile.Close();
@@ -346,7 +349,7 @@ OperationStatus CEditDoc::CopyCurrentFile(const wchar_t* pwszDestination) {
 			m_fomMode = FOM_ASREADONLY;
 		else
 			return OPS_CANNOTOPEN;
-	} else if(!m_oFile.Open(m_strPathName, nShareMode | CFile::modeReadWrite | CFile::modeNoTruncate, false)) {
+	} else if(!m_oFile.Open(m_strPathName, nReopenMode, false)) {
 		if(m_oFile.Open(m_strPathName.c_str(), CFile::modeRead | CFile::modeNoTruncate, false)) {
 			m_fomMode = FOM_ASREADONLY;
 			if(!m_bReadOnly) {
@@ -389,7 +392,7 @@ OperationStatus CEditDoc::DeleteCurrentFile() {
 
 	m_oFile.Close();
 	::SHFileOperationW(&shfos);
-	if(::PathFileExistsW(wszPath)) {	// s (SHFileExistsW ̖߂l͂ǂMpł)
+	if(toBoolean(::PathFileExistsW(wszPath))) {	// s (SHFileExistsW ̖߂l͂ǂMpł)
 		if(shfos.fAnyOperationsAborted)
 			ops = OPS_ABORTED;
 
@@ -431,18 +434,18 @@ CCharPos CEditDoc::DeleteText(CObject* pSender,
 
 	const CCharPos&	posABegin = min(posBegin, posEnd);	// n_
 	const CCharPos&	posAEnd = max(posBegin, posEnd);	// I_
-	wstring			strDeleted;							// 폜 (sɂȂƂ݂͌̉s})
+	string_t		strDeleted;							// 폜 (sɂȂƂ݂͌̉s})
 
 	if(posBegin.m_iLine == posEnd.m_iLine) {	// Ώۂ1sȓ
 		const CEditDocLine*	pLine = GetLineInfo(posEnd.m_iLine);
-		wstring&			strLine = const_cast<wstring&>(GetLine(posEnd.m_iLine));
+		string_t&			strLine = const_cast<string_t&>(GetLine(posEnd.m_iLine));
 
 		++const_cast<CEditDocLine*>(pLine)->m_cOperationHistory;
 		strDeleted = strLine.substr(posABegin.m_iChar, posAEnd.m_iChar - posABegin.m_iChar);
 		strLine.erase(posABegin.m_iChar, posAEnd.m_iChar - posABegin.m_iChar);
 	} else {							// Ώۂs
-		wstring&	strLine = const_cast<wstring&>(GetLine(posABegin.m_iLine));
-		wstring	strTail;
+		string_t&	strLine = const_cast<string_t&>(GetLine(posABegin.m_iLine));
+		string_t	strTail;
 		strDeleted = strLine.substr(posABegin.m_iChar);
 		strLine.erase(posABegin.m_iChar);
 
@@ -452,7 +455,7 @@ CCharPos CEditDoc::DeleteText(CObject* pSender,
 		BreakType					btLastLine;
 		strDeleted += CEditDoc::GetBreakString(it->m_breakType);
 		itNext = ++it;
-		for(unsigned long iLine = posABegin.m_iLine + 1; iLine < posAEnd.m_iLine + 1; ++iLine) {
+		for(length_t iLine = posABegin.m_iLine + 1; iLine < posAEnd.m_iLine + 1; ++iLine) {
 			strDeleted +=
 				((iLine != posAEnd.m_iLine) ? it->m_strLine : it->m_strLine.substr(0, posAEnd.m_iChar));
 			if(iLine != posAEnd.m_iLine)
@@ -491,26 +494,23 @@ CCharPos CEditDoc::DeleteText(CObject* pSender,
 		m_oUpdateInfo.posResult = posABegin;
 		UpdateAllViews(pSender, 0, this);
 	}
-	if(m_pEventListener != 0)
-		m_pEventListener->OnDocumentModified();
+	for_each(m_setEventListeners.begin(),
+		m_setEventListeners.end(), mem_fun(IEditDocEventListener::OnDocumentModified));
+//	for(set<IEditDocEventListener*>::iterator it
+//			= m_setEventListeners.begin(); it != m_setEventListeners.end(); ++it)
+//		(*it)->OnDocumentModified();
 
 	return posABegin;
 }
 
-///	AhDO[v̎WI
-void CEditDoc::EndEditCollection() {
-	AssertValid();
-	m_ugsGroupingState = UGS_NONE;
-}
-
 /**
  *	Ss (hLgS) 擾
  *	@param strText	i[
  */
-void CEditDoc::GetAllLines(wstring& strText) const {
+void CEditDoc::GetAllLines(string_t& strText) const {
 	AssertValid();
 	LineIterator	it;
-	unsigned long	cchAllLines = 0;
+	length_t		cchAllLines = 0;
 
 	for(it = m_listLines.begin(); it != m_listLines.end(); ++it)
 		cchAllLines += it->m_strLine.length() + CEditDoc::GetBreakString(it->m_breakType).length();
@@ -520,41 +520,41 @@ void CEditDoc::GetAllLines(wstring& strText) const {
 }
 
 ///	hLg̕ҏW̕ԂBsɂ݂͌̉sR[hgp
-unsigned long CEditDoc::GetDocumentLength() const {
+length_t CEditDoc::GetDocumentLength() const {
 	AssertValid();
 
-	unsigned long	len = 0;
+	length_t		cch = 0;
 	LineIterator	it;
 	int				cchBreak = (m_breakType == BT_CRLF) ? 2 : 1;
 
 	for(it = m_listLines.begin(); it != m_listLines.end(); ++it)
-		len += it->m_strLine.length() + cchBreak;
-	len -= cchBreak;
+		cch += it->m_strLine.length() + cchBreak;
+	cch -= cchBreak;
 
-	return len;
+	return cch;
 }
 
 ///	ҏW\ȓpsCe[^̎擾
-CEditDocLineList::iterator CEditDoc::GetInternalLineIterator(unsigned long iLine) const throw(out_of_range) {
+CEditDocLineList::iterator CEditDoc::GetInternalLineIterator(length_t iLine) const throw(out_of_range) {
 	AssertValid();
 
 	CEditDocLineList&			refList = const_cast<CEditDoc*>(this)->m_listLines;
-	CEditDocLineList::size_type	i, len = m_listLines.size();
+	length_t					i, cch = m_listLines.size();
 	CEditDocLineList::iterator	it;
 
-	if(iLine >= len)
+	if(iLine >= cch)
 		throw out_of_range("First argument is greater than list ubound.");
 
 	if(m_itCache_ == refList.end()) {	// LbVgȂꍇ
-		if(iLine <= len / 2)
+		if(iLine <= cch / 2)
 			for(i = 0, it = refList.begin(); i < iLine; ++i, ++it);
 		else
-			for(i = len, it = refList.end(); i > iLine; --i, --it);
+			for(i = cch, it = refList.end(); i > iLine; --i, --it);
 	} else {
 		if(iLine < dif(m_iLineCache_, iLine))
 			for(i = 0, it = refList.begin(); i < iLine; ++i, ++it);
-		else if(len - iLine < dif(m_iLineCache_, iLine))
-			for(i = len, it = refList.end(); i > iLine; --i, --it);
+		else if(cch - iLine < dif(m_iLineCache_, iLine))
+			for(i = cch, it = refList.end(); i > iLine; --i, --it);
 		else if(m_iLineCache_ < iLine)
 			for(i = m_iLineCache_, it = m_itCache_; i < iLine; ++i, ++it);
 		else
@@ -566,73 +566,47 @@ CEditDocLineList::iterator CEditDoc::GetInternalLineIterator(unsigned long iLine
 	return it;
 }
 
-/**
- *	w肵s̃eLXg擾B
- *	s݂Ȃ <code>out_of_range</code> 𓊂
- */
-const wstring& CEditDoc::GetLine(unsigned long iLine) const throw(out_of_range) {
-	AssertValid();
-
-	try {
-		return GetLineInfo(iLine)->m_strLine;
-	} catch(out_of_range& e) {
-		throw e;
-	}
-}
-
 /**
  *	ws牽ڂԂ
  *	@param iLine			ׂs
  *	@param bIncludeBreak	s𕶎ƂĐ邩ǂ
  *	@throw out_of_range		<var>iLine</var> ȂꍇX[
  */
-unsigned long CEditDoc::GetLineIndex(unsigned long iLine, bool bIncludeBreak) const throw(out_of_range) {
+length_t CEditDoc::GetLineIndex(length_t iLine, bool bIncludeBreak) const throw(out_of_range) {
 	AssertValid();
 
 	if(iLine >= m_listLines.size())
 		throw out_of_range("Specified line not found.");
 
-	unsigned long						iOffset = 0;
+	length_t							iOffset = 0;
 	list<CEditDocLine>::const_iterator	it = m_listLines.begin();
-	for(unsigned long i = 0; i < iLine; ++it, ++i) {
+	for(length_t i = 0; i < iLine; ++it, ++i) {
 		iOffset += it->m_strLine.length();
 		iOffset += bIncludeBreak ? CEditDoc::GetBreakString(it->m_breakType).length() : 0;
 	}
-
 	return iOffset;
 }
 
-///	sꊇĎ擾
-const CEditDocLine* CEditDoc::GetLineInfo(unsigned long iLine) const throw(out_of_range) {
-	AssertValid();
-
-	try {
-		return &(*GetLineIterator(iLine));
-	} catch(out_of_range& e) {
-		throw e;
-	}
-}
-
 ///	sCe[^̎擾
-LineIterator CEditDoc::GetLineIterator(unsigned long iLine) const throw(out_of_range) {
+LineIterator CEditDoc::GetLineIterator(length_t iLine) const throw(out_of_range) {
 	AssertValid();
 
-	CEditDocLineList::size_type	i, len = m_listLines.size();
-	LineIterator				it;
+	length_t		i, cch = m_listLines.size();
+	LineIterator	it;
 
-	if(iLine >= len)
+	if(iLine >= cch)
 		throw out_of_range("First argument is greater than list ubound.");
 
 	if(m_itCache == m_listLines.end()) {	// LbVgȂꍇ
-		if(iLine <= len / 2)
+		if(iLine <= cch / 2)
 			for(i = 0, it = m_listLines.begin(); i < iLine; ++i, ++it);
 		else
-			for(i = len, it = m_listLines.end(); i > iLine; --i, --it);
+			for(i = cch, it = m_listLines.end(); i > iLine; --i, --it);
 	} else {
 		if(iLine < dif(m_iLineCache, iLine))
 			for(i = 0, it = m_listLines.begin(); i < iLine; ++i, ++it);
-		else if(len - iLine < dif(m_iLineCache, iLine))
-			for(i = len, it = m_listLines.end(); i > iLine; --i, --it);
+		else if(cch - iLine < dif(m_iLineCache, iLine))
+			for(i = cch, it = m_listLines.end(); i > iLine; --i, --it);
 		else if(m_iLineCache < iLine)
 			for(i = m_iLineCache, it = m_itCache; i < iLine; ++i, ++it);
 		else
@@ -644,19 +618,6 @@ LineIterator CEditDoc::GetLineIterator(unsigned long iLine) const throw(out_of_r
 	return it;
 }
 
-/**
- *	w肵s̒擾B
- *	s݂Ȃ <code>out_of_range</code> 𓊂
- */
-unsigned long CEditDoc::GetLineLength(unsigned long iLine) const throw(out_of_range) {
-	AssertValid();
-	try {
-		return GetLine(iLine).length();
-	} catch(out_of_range& e) {
-		throw e;
-	}
-}
-
 ///	hLg
 void CEditDoc::InitiateDocument() {
 	AssertValid();
@@ -670,8 +631,11 @@ void CEditDoc::InitiateDocument() {
 	m_ugsGroupingState = UGS_NONE;
 	m_bOnceUndoBufferCleared = false;
 
-	if(m_pEventListener != 0)
-		m_pEventListener->OnDocumentModified();
+	for_each(m_setEventListeners.begin(),
+		m_setEventListeners.end(), mem_fun(IEditDocEventListener::OnDocumentModified));
+//	for(set<IEditDocEventListener*>::iterator it
+//			= m_setEventListeners.begin(); it != m_setEventListeners.end(); ++it)
+//		(*it)->OnDocumentModified();
 }
 
 /**
@@ -688,17 +652,17 @@ void CEditDoc::InitiateDocument() {
  *	@throw EDocumentIsReadOnly	ǂݎp̂ƂX[
  */
 CCharPos CEditDoc::InsertText(CObject* pSender,
-		const CCharPos& pos, const wstring& strText) throw(EDocumentIsReadOnly) {
+		const CCharPos& pos, const string_t& strText) throw(EDocumentIsReadOnly) {
 	AssertValid();
 
 	if(m_bReadOnly)
 		throw EDocumentIsReadOnly();
 
-	wstring::size_type	iLast = strText.find_first_of(CEditDoc::m_wszBreakChars);
-	wstring::size_type	iNext;
+	length_t			iLast = strText.find_first_of(CEditDoc::m_wszBreakChars);
+	length_t			iNext;
 	CCharPos			posResult;
 	const CEditDocLine*	pLastLine = GetLineInfo(pos.m_iLine);
-	wstring&			strLine = const_cast<wstring&>(pLastLine->m_strLine);
+	string_t&			strLine = const_cast<wstring&>(pLastLine->m_strLine);
 
 	if(iLast == wstring::npos) {	// ͂ɉsꍇ
 		strLine.insert(pos.m_iChar, strText);
@@ -706,9 +670,9 @@ CCharPos CEditDoc::InsertText(CObject* pSender,
 		posResult.m_iLine = pos.m_iLine;
 		posResult.m_iChar = pos.m_iChar + strText.length();
 	} else {	// ͂ɉsꍇ
-		unsigned long	iLine = pos.m_iLine;
+		length_t		iLine = pos.m_iLine;
 		CEditDocLineList::iterator	it = GetInternalLineIterator(pos.m_iLine);
-		wstring			strInsert = strText + it->m_strLine.substr(pos.m_iChar);
+		string_t		strInsert = strText + it->m_strLine.substr(pos.m_iChar);
 		BreakType		bt;
 		BreakType		btFirstLine = it->m_breakType;	// 擪s̉s (}Ԍɕt)
 
@@ -775,8 +739,11 @@ CCharPos CEditDoc::InsertText(CObject* pSender,
 	if(m_ugsGroupingState == UGS_WAITFORFIRSTEDIT)
 		m_ugsGroupingState = UGS_WAITFORCONTINUEEDIT;
 
-	if(m_pEventListener != 0)
-		m_pEventListener->OnDocumentModified();
+	for_each(m_setEventListeners.begin(),
+		m_setEventListeners.end(), mem_fun(IEditDocEventListener::OnDocumentModified));
+//	for(set<IEditDocEventListener*>::iterator it
+//			= m_setEventListeners.begin(); it != m_setEventListeners.end(); ++it)
+//		(*it)->OnDocumentModified();
 	if(!m_bIgnoreViews) {
 		// Ō̍XVXV
 		m_oUpdateInfo.usSummary = US_OPERATION_INSERT;
@@ -793,31 +760,31 @@ CCharPos CEditDoc::InsertText(CObject* pSender,
  *	hLgǂݍ
  *	@param strPathName	t@CpX
  *	@param fom			r[hȂ
- *	@param nCodePage	R[hy[W (ȗƎ)
+ *	@param cp			R[hy[W (ȗƎ)
  *	@return				
  */
 StreamStatus CEditDoc::LoadDocument(const wstring& strPathName,
-		FileOpenMode fom, UINT nCodePage /* = EXTXP_AUTO */) {
+		FileOpenMode fom, CodePage cp /* = CPEX_LOCALE_AUTODETECT */) {
 //	CTimer tm(L"LoadDocument");	// 2.86s / 1MB
 	AssertValid();
 
-	if(nCodePage != EXTCP_AUTO && !IsAvailableCodePage(nCodePage))
+	if(!CEncoder::IsValidCodePage(cp))
 		return SS_ILLEGALCODEPAGE;
 
 	CWaitCursor		wc;
 	StreamStatus	ss = SS_OK;
 	wchar_t			wszPathName[MAX_PATH];
 	HGLOBAL			hGlobal = 0, hGlobal2 = 0;
-	unsigned char*	pszBuffer = 0;		// obt@
-	wchar_t*		pwszBuffer = 0;
-	wchar_t*		pwszFirstBreak = 0;	// ŏɌꂽs
+	char*			pszBuffer = 0;		// obt@
+	char_t*			pwszBuffer = 0;
+	char_t*			pwszFirstBreak = 0;	// ŏɌꂽs
 	DWORD			dwFileSize;			// t@CTCY
 	DWORD			dwRead;				// ۂɓǂݎf[^
 	CCharPos		posLast(0, 0);		// ɕǉʒu
 
 	// V[gJbg̉
 	const wchar_t*	pwszExtension = ::PathFindExtensionW(strPathName.c_str());
-	if(pwszExtension != 0 && (
+	if(wcslen(pwszExtension) != 0 && (
 			(::StrCmpIW(pwszExtension + 1, L"lnk") == 0)
 			/*|| (::StrCmpIW(pwszExtension + 1, L"url") == 0)*/)) {
 		CComPtr<IShellLink>		pShellLink;
@@ -851,11 +818,11 @@ StreamStatus CEditDoc::LoadDocument(const wstring& strPathName,
 		m_oFile.GetFileTime(0, 0, &m_lastWriteTime);
 		dwFileSize = m_oFile.GetFileSize(0);	// 4GB ̃XNvgȂ񂩖낤
 		hGlobal = ::GlobalAlloc(GMEM_MOVEABLE, dwFileSize);
-		pszBuffer = static_cast<unsigned char*>(::GlobalLock(hGlobal));
+		pszBuffer = static_cast<char*>(::GlobalLock(hGlobal));
 
 		if(dwFileSize != 0)
 			dwRead = m_oFile.Read(pszBuffer, dwFileSize);
-	} catch(CFileException& e) {
+	} catch(CFileException&) {
 		if(pszBuffer != 0)
 			::GlobalUnlock(hGlobal);
 		if(hGlobal != 0)
@@ -866,73 +833,27 @@ StreamStatus CEditDoc::LoadDocument(const wstring& strPathName,
 	m_oFile.Close();
 
 	// R[hϊ
-	CharCode	cc;
 	if(dwFileSize != 0) {
-		hGlobal2 = ::GlobalAlloc(GMEM_MOVEABLE, sizeof(wchar_t) * dwRead * 3);	// ȃTCY...
+		cp = CEncoder::DetectCodePage(pszBuffer, std::min(dwRead, 4UL * 1024), cp);	// xƐM̃g[h͂
+
+		CEncoder*				pEncoder = CEncoder::Create(cp);
+		assert(pEncoder != 0);
+		std::size_t				cchDest = 1 * dwRead;
+		const ByteOrderMark*	pBom = pEncoder->GetByteOrderMark();
+
+		hGlobal2 = ::GlobalAlloc(GMEM_MOVEABLE, (cchDest + 1) * sizeof(char_t));
 		pwszBuffer = static_cast<wchar_t*>(::GlobalLock(hGlobal2));
-		switch(nCodePage) {
-		case EXTCP_AUTO:
-			cc = CC_AUTO;
-			dwRead = ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
-			switch(cc) {
-			case CC_UTF16LE:	nCodePage = EXTCP_UTF16LE;	break;
-			case CC_UTF16BE:	nCodePage = EXTCP_UTF16BE;	break;
-			case CC_UTF32LE:	nCodePage = EXTCP_UTF32LE;	break;
-			case CC_UTF32BE:	nCodePage = EXTCP_UTF32BE;	break;
-			case CC_UTF8:		nCodePage = CP_UTF8;		break;
-			case CC_UTF7:		nCodePage = EXTCP_UTF5;		break;
-			case CC_SHIFTJIS:	nCodePage = 932;			break;
-			case CC_JIS:		nCodePage = 50221;			break;
-			case CC_EUCJP:		nCodePage = EXTCP_EUCJP;	break;
-			}
-			break;
-		case EXTCP_UTF16LE:
-			cc = CC_UTF16LE;
-			dwRead = ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
-			break;
-		case EXTCP_UTF16BE:
-			cc = CC_UTF16BE;
-			dwRead = ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
-			break;
-		case EXTCP_UTF32LE:
-			cc = CC_UTF32LE;
-			dwRead = ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
-			break;
-		case EXTCP_UTF32BE:
-			cc = CC_UTF32BE;
-			dwRead = ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
-			break;
-		case CP_UTF8:
-			cc = CC_UTF8;
-			dwRead = ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
-			break;
-		case EXTCP_UTF5:
-			cc = CC_UTF5;
-			dwRead = ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
-			break;
-		case 932:	// Shift-JIS
-			cc = CC_SHIFTJIS;
-			dwRead = ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
-			break;
-		case 50221:	// JIS
-			cc = CC_JIS;
-			dwRead = ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
-			break;
-		case EXTCP_EUCJP:
-			cc = CC_EUCJP;
-			dwRead = ConvertToUnicode(cc, pwszBuffer, pszBuffer, dwRead);
-			break;
-		default:
-			dwRead = ::MultiByteToWideChar(nCodePage, 0,
-				reinterpret_cast<const char*>(pszBuffer), dwRead, pwszBuffer, dwRead * 2);
-			break;
-		}
-		*(pwszBuffer + dwRead) = 0;
+		if(pBom->cBytes != 0 && dwRead >= pBom->cBytes && strncmp(pszBuffer, pBom->pszBOM, pBom->cBytes) == 0)
+			cchDest = pEncoder->ConvertToUnicode(pwszBuffer, cchDest, pszBuffer + pBom->cBytes, dwRead - pBom->cBytes);
+		else
+			cchDest = pEncoder->ConvertToUnicode(pwszBuffer, cchDest, pszBuffer, dwRead);
+		delete pEncoder;
+		*(pwszBuffer + cchDest) = 0;
 
 		// sƂɋ؂AXgɒǉĂ
-		unsigned long	iNext, iLast = 0;
-		wchar_t*		pwszBreak;
-		BreakType		bt;
+		length_t	iNext, iLast = 0;
+		char_t*		pwszBreak;
+		BreakType	bt;
 		m_listLines.clear();
 		while(true) {
 			pwszBreak = wcspbrk(pwszBuffer + iLast, CEditDoc::m_wszBreakChars);
@@ -951,12 +872,12 @@ StreamStatus CEditDoc::LoadDocument(const wstring& strPathName,
 				m_listLines.push_back(CEditDocLine(wstring(pwszBuffer + iLast, iNext - iLast), bt));
 				iLast = iNext + ((bt != BT_CRLF) ? 1 : 2);
 			} else {	// ŏIs
-				m_listLines.push_back(CEditDocLine(wstring(pwszBuffer + iLast, dwRead - iLast)));
+				m_listLines.push_back(CEditDocLine(wstring(pwszBuffer + iLast, cchDest - iLast)));
 				break;
 			}
 		}
 	} else {	// ̃t@C
-		nCodePage = ::GetACP();
+		cp = ::GetACP();
 		m_listLines.clear();
 		m_listLines.push_back(CEditDocLine());
 		InsertText(0, CCharPos(0, 0), L"");
@@ -995,14 +916,14 @@ StreamStatus CEditDoc::LoadDocument(const wstring& strPathName,
 			m_oFile.Open(wszPathName, CFile::modeRead | CFile::shareDenyNone);
 		else if(fom == FOM_DENYWRITE)
 			m_oFile.Open(wszPathName, CFile::modeRead | CFile::shareDenyWrite);
-		else	/* fom == FOM_DENYWRITE */
+		else	/* if(fom == FOM_DENYREAD) */
 			m_oFile.Open(wszPathName,
 				CFile::modeRead | CFile::modeWrite | CFile::shareDenyRead | CFile::shareDenyWrite);
-	} catch(CFileException& e) {
+	} catch(CFileException&) {
 		ss = SS_LOCKDENIED;
 	}
 
-	m_nCodePage = nCodePage;
+	m_nCodePage = cp;
 	ClearUndoBuffer();
 	m_bModified = false;
 	m_ugsGroupingState = UGS_NONE;
@@ -1012,8 +933,11 @@ StreamStatus CEditDoc::LoadDocument(const wstring& strPathName,
 
 	SetupAllViews();	// r[ɒʒm
 
-	if(m_pEventListener != 0)
-		m_pEventListener->OnDocumentModified();
+	for_each(m_setEventListeners.begin(),
+		m_setEventListeners.end(), mem_fun(IEditDocEventListener::OnDocumentModified));
+//	for(set<IEditDocEventListener*>::iterator it
+//			= m_setEventListeners.begin(); it != m_setEventListeners.end(); ++it)
+//		(*it)->OnDocumentModified();
 
 	return ss;
 }
@@ -1050,9 +974,6 @@ OperationStatus CEditDoc::MoveCurrentFile(const wchar_t* pwszDestination) {
 	AssertValid();
 	assert(pwszDestination != 0);
 
-	OperationStatus	ops = OPS_OK;
-	UINT			nShareMode = 0;
-
 	if(m_strPathName.empty())
 		return OPS_HASNOINSTANCE;
 	if(::PathFileExistsW(pwszDestination))
@@ -1060,11 +981,14 @@ OperationStatus CEditDoc::MoveCurrentFile(const wchar_t* pwszDestination) {
 	if(m_bReadOnly)
 		return OPS_MODEISREADONLY;
 
+	OperationStatus	ops = OPS_OK;
+	UINT			nReopenMode = CFile::modeNoTruncate;
+
 	switch(static_cast<int>(m_fomMode)) {
-		case FOM_DENYWRITE:	nShareMode = CFile::shareDenyWrite;	break;
-		case FOM_DENYREAD:	nShareMode = CFile::shareDenyRead;	break;
+		case FOM_DENYWRITE:		nReopenMode |= CFile::modeRead | CFile::shareDenyWrite;		break;
+		case FOM_DENYREAD:		nReopenMode |= CFile::modeReadWrite | CFile::shareDenyRead;	break;
 		case FOM_DENYNONE:
-		case FOM_ASREADONLY:nShareMode = CFile::shareDenyNone;	break;
+		case FOM_ASREADONLY:	nReopenMode |= CFile::modeRead | CFile::shareDenyNone;		break;
 	}
 
 	m_oFile.Close();
@@ -1074,7 +998,8 @@ OperationStatus CEditDoc::MoveCurrentFile(const wchar_t* pwszDestination) {
 		m_strPathName = pwszDestination;
 		m_strTitle = ::PathFindFileNameW(pwszDestination);
 	}
-	if(!m_oFile.Open(m_strPathName, nShareMode | CFile::modeReadWrite | CFile::modeNoTruncate, false)) {
+	// bN悤Ƃ
+	if(!m_oFile.Open(m_strPathName, nReopenMode, false)) {
 		if(m_oFile.Open(m_strPathName.c_str(), CFile::modeRead | CFile::modeNoTruncate, false)) {
 			m_fomMode = FOM_ASREADONLY;
 			if(!m_bReadOnly) {
@@ -1105,6 +1030,15 @@ void CEditDoc::Redo() throw(EDocumentIsReadOnly) {
 	EndEditCollection();
 	m_oUpdateInfo.usSummary = US_END_UNDOGROUP;
 	UpdateAllViews(this, 0, this);
+
+	if(!m_pUndoManager->IsModifiedSinceLastSave()) {
+		m_bModified = false;
+		for_each(m_setEventListeners.begin(),
+			m_setEventListeners.end(), mem_fun(IEditDocEventListener::OnDocumentModified));
+//		for(set<IEditDocEventListener*>::iterator it
+//				= m_setEventListeners.begin(); it != m_setEventListeners.end(); ++it)
+//			(*it)->OnDocumentModified();
+	}
 }
 
 /**
@@ -1112,43 +1046,41 @@ void CEditDoc::Redo() throw(EDocumentIsReadOnly) {
  *	@param strPathName		t@C̃pX
  *	@param sdo				IvV
  *	@param bt				ꂷsR[h
- *	@param nCodePage		R[hy[WBȗƌ݂̃R[hy[Wgp
+ *	@param cp				R[hy[WBȗƌ݂̃R[hy[Wgp
  *	@return					݌ʁB SS_OK
  */
 StreamStatus CEditDoc::SaveDocument(const wstring& strPathName,
-		SaveDocumentOption sdo, BreakType bt, UINT nCodePage /* = EXTCP_AUTO */) {
+		SaveDocumentOption sdo, BreakType bt, UINT cp /* = EXCP_AUTODETECT */) {
 	AssertValid();
 
 	// R[hy[WCXg[Ă邩
-	if(nCodePage != EXTCP_AUTO && !IsAvailableCodePage(nCodePage))
+	if(!CEncoder::IsValidCodePage(cp))
 		return SS_ILLEGALCODEPAGE;
-	if(nCodePage == EXTCP_AUTO)	// ݂̃R[hy[Wg
-		nCodePage = m_nCodePage;
+	if(CEncoder::IsCodePageForAutoDetection(cp))	// ݂̃R[hy[Wg
+		cp = m_nCodePage;
 
 	// ϊR[hy[Wŕ\łȂ邩
-	if(!toBoolean(sdo & SDO_IGNORE_NOFITCHARS)
-			&& nCodePage != EXTCP_UTF16LE
-			&& nCodePage != EXTCP_UTF16BE
-			&& nCodePage != EXTCP_UTF32LE
-			&& nCodePage != EXTCP_UTF32BE
-			&& nCodePage != EXTCP_UTF5
-			&& nCodePage != 50221) {
+/*	if(!toBoolean(sdo & SDO_IGNORE_NOFITCHARS)
+			&& cp != EXTCP_UTF16LE
+			&& cp != EXTCP_UTF16BE
+			&& cp != EXTCP_UTF32LE
+			&& cp != EXTCP_UTF32BE
+			&& cp != EXTCP_UTF5
+			&& cp != 50221) {
 		BOOL	bUsedDefaultChar = FALSE;
 		wstring	strText;
 		GetAllLines(strText);
 		::WideCharToMultiByte(nCodePage, 0, strText.c_str(), strText.length(), 0, 0, 0, &bUsedDefaultChar);
 		if(toBoolean(bUsedDefaultChar))
 			return SS_WRITE_CONTAININVALIDCHAR;
-	}
+	}*/
 
 	CWaitCursor		wc;
 	StreamStatus	ss = SS_OK;
-	HGLOBAL			hGlobal = 0;
-	char*			pszBuffer = 0;
 
 	// fobO͏ɃobNAbv (㏑̏ꍇ̂)
 #ifdef _DEBUG
-	if(::PathFileExistsW(strPathName.c_str())) {
+	if(toBoolean(::PathFileExistsW(strPathName.c_str()))) {
 		wchar_t	wszBackupPath[MAX_PATH + 1];
 		SHFILEOPSTRUCTW	shfos = {
 			0, FO_DELETE, wszBackupPath, 0, FOF_ALLOWUNDO | FOF_NOCONFIRMATION | FOF_SILENT, false, 0
@@ -1166,36 +1098,23 @@ StreamStatus CEditDoc::SaveDocument(const wstring& strPathName,
 		if(m_oFile.IsOpened())
 			m_oFile.Close();
 		m_oFile.Open(strPathName, CFile::modeReadWrite | CFile::shareDenyWrite | CFile::modeCreate);
-	} catch(CFileException& e) {	// ߂Ȃꍇ͈ȑÕt@CJȂ
+	} catch(CFileException&) {	// ߂Ȃꍇ͈ȑÕt@CJȂ
 		m_oFile.Open(m_strPathName, (m_bReadOnly ? CFile::modeRead : CFile::modeReadWrite)
 			| CFile::shareDenyWrite | CFile::modeCreate | CFile::modeNoTruncate);
 		return SS_UNKNOWNERROR;
 	}
 
+	CEncoder*	pEncoder = CEncoder::Create(cp);
+	HGLOBAL		hGlobal = 0;
+	char*		pszBuffer = 0;
+
+	assert(pEncoder != 0);
+
 	// BOM
 	if(toBoolean(sdo & SDO_WRITE_BOM)) {
-		if(nCodePage == EXTCP_UTF16LE) {
-			unsigned char	pszBOM[] = {0xFF, 0xFE};
-			m_oFile.Write(pszBOM, 2);
-		} else if(nCodePage == EXTCP_UTF16BE) {
-			unsigned char	pszBOM[] = {0xFE, 0xFF};
-			m_oFile.Write(pszBOM, 2);
-		} else if(nCodePage == CP_UTF8) {
-			unsigned char	pszBOM[] = {0xEF, 0xBB, 0xBF};
-			m_oFile.Write(pszBOM, 3);
-		} else if(nCodePage == EXTCP_UTF32LE) {
-			unsigned char	pszBOM[] = {0xFF, 0xFE, 0x00, 0x00};
-			m_oFile.Write(pszBOM, 4);
-		} else if(nCodePage == EXTCP_UTF32BE) {
-			unsigned char	pszBOM[] = {0x00, 0x00, 0xFE, 0xFF};
-			m_oFile.Write(pszBOM, 4);
-		} else if(nCodePage == 50225) {	// ISO-2022-KR
-			unsigned char	pszBOM[] = {0x1B, 0x24, 0x29, 0x43};
-			m_oFile.Write(pszBOM, 4);
-		} else if(nCodePage == 50227 || nCodePage == 50229) {	// ISO-2022-CN
-			unsigned char	pszBOM[] = {0x1B, 0x24, 0x29, 0x41};
-			m_oFile.Write(pszBOM, 4);
-		}
+		const ByteOrderMark*	pBom = pEncoder->GetByteOrderMark();
+		if(pBom->cBytes > 0)
+			m_oFile.Write(pBom->pszBOM, pBom->cBytes);
 	}
 
 	// sR[ȟ
@@ -1204,13 +1123,12 @@ StreamStatus CEditDoc::SaveDocument(const wstring& strPathName,
 
 	// 1sϊĂ珑
 	CEditDocLineList::iterator	it = m_listLines.begin();
-	unsigned long				cLine = GetLineCount();	// s
-	unsigned long				cchLine;				// R[hϊO̒
-	unsigned long				len;					// R[hϊ̒
+	length_t					cLine = GetLineCount();	// s
+	length_t					cchLine;				// R[hϊO̒
+	length_t					len;					// R[hϊ̒
 	wstring						strLine;				// 1s
-	CharCode					cc;
 
-	for(unsigned long iLine = 0; iLine < cLine; ++iLine, ++it) {
+	for(length_t iLine = 0; iLine < cLine; ++iLine, ++it) {
 		it->m_cOperationHistory = 0;	// 엚
 		if(bt != BT_AUTO)
 			it->m_breakType = bt;		// sR[h㏑
@@ -1225,74 +1143,17 @@ StreamStatus CEditDoc::SaveDocument(const wstring& strPathName,
 				break;
 		} else
 			cchLine += (((bt == BT_AUTO) ? it->m_breakType : m_breakType) == BT_CRLF) ? 2 : 1;
-		switch(nCodePage) {
-		case EXTCP_UTF16LE:
-			cc = CC_UTF16LE;
-			hGlobal = ::GlobalAlloc(GHND, cchLine * sizeof(wchar_t) * 2);
-			pszBuffer = reinterpret_cast<char*>(::GlobalLock(hGlobal));
-			len = ConvertUnicodeTo(cc, pszBuffer, strLine.c_str(), 0, cchLine);
-			break;
-		case EXTCP_UTF16BE:
-			cc = CC_UTF16BE;
-			hGlobal = ::GlobalAlloc(GHND, cchLine * sizeof(wchar_t) * 2);
-			pszBuffer = reinterpret_cast<char*>(::GlobalLock(hGlobal));
-			len = ConvertUnicodeTo(cc, pszBuffer, strLine.c_str(), 0, cchLine);
-			break;
-		case CP_UTF8:
-			cc = CC_UTF8;
-			hGlobal = ::GlobalAlloc(GHND, cchLine * sizeof(wchar_t) * 3);
-			pszBuffer = reinterpret_cast<char*>(::GlobalLock(hGlobal));
-			len = ConvertUnicodeTo(cc, pszBuffer, strLine.c_str(), 0, cchLine);
-			break;
-		case EXTCP_UTF32LE:
-			cc = CC_UTF32LE;
-			hGlobal = ::GlobalAlloc(GHND, cchLine * sizeof(wchar_t) * 2);
-			pszBuffer = reinterpret_cast<char*>(::GlobalLock(hGlobal));
-			len = ConvertUnicodeTo(cc, pszBuffer, strLine.c_str(), 0, cchLine);
-			break;
-		case EXTCP_UTF32BE:
-			cc = CC_UTF32BE;
-			hGlobal = ::GlobalAlloc(GHND, cchLine * sizeof(wchar_t) * 2);
-			pszBuffer = reinterpret_cast<char*>(::GlobalLock(hGlobal));
-			len = ConvertUnicodeTo(cc, pszBuffer, strLine.c_str(), 0, cchLine);
-			break;
-		case EXTCP_UTF5:
-			cc = CC_UTF5;
-			hGlobal = ::GlobalAlloc(GHND, cchLine * sizeof(wchar_t) * 4);
-			pszBuffer = reinterpret_cast<char*>(::GlobalLock(hGlobal));
-			len = ConvertUnicodeTo(cc, pszBuffer, strLine.c_str(), 0, cchLine);
-			break;
-		case 932:	// Shift-JIS
-			cc = CC_SHIFTJIS;
-			hGlobal = ::GlobalAlloc(GHND, cchLine * sizeof(wchar_t) * 3);
-			pszBuffer = reinterpret_cast<char*>(::GlobalLock(hGlobal));
-			len = ConvertUnicodeTo(cc, pszBuffer, strLine.c_str(), 0, cchLine);
-			break;
-		case EXTCP_EUCJP:
-			cc = CC_EUCJP;
-			hGlobal = ::GlobalAlloc(GHND, cchLine * sizeof(wchar_t) * 2);
-			pszBuffer = reinterpret_cast<char*>(::GlobalLock(hGlobal));
-			len = ConvertUnicodeTo(cc, pszBuffer, strLine.c_str(), 0, cchLine);
-			break;
-		case 50221:	// JIS
-			cc = CC_JIS;
-			hGlobal = ::GlobalAlloc(GHND, cchLine * sizeof(wchar_t) * 3);
-			pszBuffer = reinterpret_cast<char*>(::GlobalLock(hGlobal));
-			len = ConvertUnicodeTo(cc, pszBuffer, strLine.c_str(), 0, cchLine);
-			break;
-		default:
-			len = ::WideCharToMultiByte(nCodePage, 0,
-					strLine.c_str(), cchLine, 0, 0, 0, 0);
-			hGlobal = ::GlobalAlloc(GHND, len);
-			pszBuffer = reinterpret_cast<char*>(::GlobalLock(hGlobal));
-			len = ::WideCharToMultiByte(nCodePage, 0,
-					strLine.c_str(), cchLine, pszBuffer, len, 0, 0);
-			break;
-		}
+
+		const std::size_t	cBytes = pEncoder->GetMaxCharacterLength() * cchLine;
+
+		hGlobal = ::GlobalAlloc(GHND, cBytes);
+		pszBuffer = static_cast<char*>(::GlobalLock(hGlobal));
+		len = pEncoder->ConvertFromUnicode(pszBuffer, cBytes, strLine.c_str(), cchLine);
 		m_oFile.Write(pszBuffer, len);
 		::GlobalUnlock(hGlobal);
 		::GlobalFree(hGlobal);
 	}
+	delete pEncoder;
 
 	m_oFile.Close();
 
@@ -1302,14 +1163,15 @@ StreamStatus CEditDoc::SaveDocument(const wstring& strPathName,
 			if(!LockCurrentFile(m_fomMode))
 				ss = SS_LOCKDENIED;
 		}
-	} catch(CFileException& e) {
+	} catch(CFileException&) {
 		ss = SS_LOCKDENIED;
 	}
 
+	m_pUndoManager->OnSave();
 	m_bModified = false;
 	m_bReadOnly = false;
 	m_breakType = (bt != BT_AUTO) ? bt : m_breakType;
-	m_nCodePage = nCodePage;
+	m_nCodePage = cp;
 	m_strPathName = strPathName;
 	m_strTitle = m_strPathName.substr(m_strPathName.rfind(L'\\') + 1);
 
@@ -1325,8 +1187,11 @@ StreamStatus CEditDoc::SaveDocument(const wstring& strPathName,
 	m_oUpdateInfo.usSummary = US_SAVEDOCUMENT;
 	UpdateAllViews(this, 0, this);
 
-	if(m_pEventListener != 0)
-		m_pEventListener->OnDocumentModified();
+	for_each(m_setEventListeners.begin(),
+		m_setEventListeners.end(), mem_fun(IEditDocEventListener::OnDocumentModified));
+//	for(set<IEditDocEventListener*>::iterator it
+//			= m_setEventListeners.begin(); it != m_setEventListeners.end(); ++it)
+//		(*it)->OnDocumentModified();
 	return ss;
 }
 
@@ -1379,8 +1244,8 @@ bool CEditDoc::SendCurrentFile(bool bAsAttachment, bool bShowDialog /* = true */
 		fileDesc.lpszPathName = szFilePath;
 		fileDesc.nPosition = static_cast<unsigned long>(-1);
 	} else {	// {ƂđMƂ
-		unsigned long	i = 0;
-		unsigned long	iLine = 0;
+		length_t		i = 0;
+		length_t		iLine = 0;
 		LineIterator	it;
 		CEditDocLine	oLine;
 
@@ -1424,61 +1289,10 @@ bool CEditDoc::SendCurrentFile(bool bAsAttachment, bool bShowDialog /* = true */
 	return true;
 }
 
-/**
- *	sR[h̐ݒ
- *	@param breakType		Vݒ肷sR[h
- *	@throw invalid_argument	sR[hȂƂX[
- */
-void CEditDoc::SetBreakType(BreakType breakType) {
-	AssertValid();
-	switch(breakType) {
-	case BT_LF:
-	case BT_CR:
-	case BT_CRLF:
-	case BT_NEL:
-	case BT_LS:
-	case BT_PS:
-		m_breakType = breakType;
-		break;
-	default:
-		throw invalid_argument("Specified break type is invalid.");
-	}
-}
-
-/**
- *	R[hy[W̐ݒ
- *	@param nCodePage		Vݒ肷R[hy[Wԍ
- *	@throw invalid_argument	R[hy[WpłȂƂX[
- */
-void CEditDoc::SetCodePage(UINT nCodePage) throw(invalid_argument) {
-	AssertValid();
-	if(!IsAvailableCodePage(nCodePage))
-		throw invalid_argument("Specified code page is not available.");
-	m_nCodePage = nCodePage;
-}
-
-/**
- *	CxgXi̐ݒ
- *	@param pEventListener	VCxgXi (null ł悢)
- */
-void CEditDoc::SetEventListener(IEditDocEventListener* pEventListener) {
-	AssertValid();
-	m_pEventListener = pEventListener;
-}
-
-///	ǂݎp[h̐ݒ
-void CEditDoc::SetReadOnly(bool bReadOnly /* = true */) {
-	AssertValid();
-	m_bReadOnly = bReadOnly;
-	if(m_pEventListener != 0)
-		m_pEventListener->OnDocumentModified();
-}
-
 ///	::SetTimer ̃R[obN֐B1000~bƂɌĂяoA
 ///	t@CvZXŕύXĂȂĎ
-void CALLBACK CEditDoc::TimerProc(HWND hWnd, UINT nMsg, UINT_PTR idEvent, DWORD dwTime) {
+void CALLBACK CEditDoc::TimerProc(HWND hWnd, UINT nMsg, TIMERPROC_3RD_PARAM_TYPE idEvent, DWORD dwTime) {
 	map<UINT, CEditDoc*>::iterator	it = CEditDoc::m_mapDocuments.find(idEvent);
-
 	if(it != CEditDoc::m_mapDocuments.end())
 		it->second->CheckFileLastWriteTime();
 }
@@ -1487,7 +1301,7 @@ void CALLBACK CEditDoc::TimerProc(HWND hWnd, UINT nMsg, UINT_PTR idEvent, DWORD
  *	AhD̎s
  *	@throw EDocumentIsReadOnly	ǂݎp̂ƂX[
  */
-void CEditDoc::Undo() {
+void CEditDoc::Undo() throw(EDocumentIsReadOnly) {
 	AssertValid();
 	if(m_bReadOnly)
 		throw EDocumentIsReadOnly();
@@ -1502,26 +1316,14 @@ void CEditDoc::Undo() {
 	m_oUpdateInfo.usSummary = US_END_UNDOGROUP;
 	UpdateAllViews(this, 0, this);
 
-	if(!m_bOnceUndoBufferCleared && m_pUndoManager->GetUndoBufferLength() == 0) {
+	if(!m_pUndoManager->IsModifiedSinceLastSave()) {
 		m_bModified = false;
-	if(m_pEventListener != 0)
-		m_pEventListener->OnDocumentModified();
+		for_each(m_setEventListeners.begin(),
+			m_setEventListeners.end(), mem_fun(IEditDocEventListener::OnDocumentModified));
+//		for(set<IEditDocEventListener*>::iterator it
+//				= m_setEventListeners.begin(); it != m_setEventListeners.end(); ++it)
+//			(*it)->OnDocumentModified();
 	}
 }
 
-///	hLgV쐬ƂɌĂяo
-bool CEditDoc::OnNewDocument() {
-	return true;
-}
-
-///	hLgǂݍ݂̂߂Ƀt@CJƂɌĂяo
-bool CEditDoc::OnOpenDocument(const wstring& strPathName) {
-	return true;
-}
-
-///	hLgۑƂɌĂяo
-bool CEditDoc::OnSaveDocument(const wstring& strPathName) {
-	return true;
-}
-
 /* [EOF] */
\ No newline at end of file
