@@ -6,8 +6,11 @@
 
 #include <stack>
 #include <map>
+#include <set>
 #include "AscensionCommon.h"
+#include "..\..\Manah\SmallObject.h"
 #include "..\..\Manah\DocumentView.h"
+#include "..\..\Manah\Encoder.h"
 #include "..\..\Manah\File.h"
 
 
@@ -16,6 +19,13 @@ namespace Ascension {
 class CEditDoc;
 class CEditController;
 
+// SetTimer ̑4̖
+#if(_MSC_VER < 1300)
+#define TIMERPROC_3RD_PARAM_TYPE	UINT
+#else
+#define TIMERPROC_3RD_PARAM_TYPE	UINT_PTR
+#endif
+
 
 // COperation interface definition
 /////////////////////////////////////////////////////////////////////////////
@@ -43,19 +53,19 @@ class CDeleteOperation;
 class COperationUnit;
 
 ///	1̑}
-class CInsertOperation : public IOperation {
+class CInsertOperation : public IOperation, public Manah::Windows::CSmallObject<> {
 
 	friend class CDeleteOperation;
 	friend class COperationUnit;
 	// f[^o
 private:
-	bool			m_bType;	// ̎ (true: }Afalse: 폜)
-	CCharPos		m_pos;		// }ʒu
-	std::wstring	m_strText;	// }
+	bool		m_bType;	// ̎ (true: }Afalse: 폜)
+	CCharPos	m_pos;		// }ʒu
+	string_t	m_strText;	// }
 
 	// RXgN^
 public:
-	CInsertOperation(const CCharPos& pos, const std::wstring& strText);
+	CInsertOperation(const CCharPos& pos, const string_t& strText);
 
 	// \bh
 public:
@@ -68,7 +78,7 @@ public:
 /////////////////////////////////////////////////////////////////////////////
 
 ///	1̍폜
-class CDeleteOperation : public IOperation {
+class CDeleteOperation : public IOperation, public Manah::Windows::CSmallObject<> {
 
 	friend class CInsertOperation;
 	friend class COperationUnit;
@@ -94,7 +104,7 @@ public:
 /////////////////////////////////////////////////////////////////////////////
 
 ///	܂Ƃ߂ăAhD/hD\ȈȂ
-class COperationUnit {
+class COperationUnit : public Manah::Windows::CSmallObject<> {
 	// RXgN^
 public:
 	virtual ~COperationUnit();
@@ -115,24 +125,6 @@ private:
 // CEditDoc class definition
 /////////////////////////////////////////////////////////////////////////////
 
-// R[hy[WXgɌȂR[h
-/// ʁA݂̕R[h
-const UINT	EXTCP_AUTO		= 65500;
-/// UTF-5
-const UINT	EXTCP_UTF5		= 65506;
-
-// 
-/// UTF-16 gGfBA
-const UINT	EXTCP_UTF16LE	= 1200;
-/// UTF-16 rbOGfBA
-const UINT	EXTCP_UTF16BE	= 1201;
-/// UTF-32 gGfBA
-const UINT	EXTCP_UTF32LE	= 12000;
-/// UTF-32 rbOGfBA
-const UINT	EXTCP_UTF32BE	= 12001;
-/// EUC-JP
-const UINT	EXTCP_EUCJP		= 51932;
-
 /// sR[h
 enum BreakType {
 	/// ϊAw薳Ȃ
@@ -242,32 +234,32 @@ struct TUpdateInfo {
 };
 
 ///	hLgǂݎp̂ƂҏW悤ƂƃX[
-class EDocumentIsReadOnly {
+class EDocumentIsReadOnly : public std::logic_error {
 public:
-	wstring	GetDescription() const {
-		return L"Document is readonly. Any edit process is denied.";
+	EDocumentIsReadOnly()
+			: std::logic_error("Document is readonly. Any edit process is denied.") {
 	}
 };
 
 ///	s̓e
-class CEditDocLine {
+class CEditDocLine : public Manah::Windows::CSmallObject<> {
 	friend class CEditDoc;
 
 	// f[^o
 private:
-	std::wstring	m_strLine;				// s
+	string_t		m_strLine;				// s
 	BreakType		m_breakType;			// s̎
 	unsigned long	m_cOperationHistory;	// AhDJE^ (0ŕύX̏)
 
 	// RXgN^
 public:
-	explicit CEditDocLine(std::wstring& strLine = wstring(L""), BreakType breakType = BT_AUTO, bool bModified = false)
+	explicit CEditDocLine(string_t& strLine = string_t(L""), BreakType breakType = BT_AUTO, bool bModified = false)
 		: m_strLine(strLine), m_breakType(breakType), m_cOperationHistory(bModified ? 1 : 0) {
 	}
 
 	// \bh
 public:
-	const std::wstring&	GetLine() const {
+	const string_t&	GetLine() const {
 		return m_strLine;
 	}
 	BreakType	GetBreakType() const {
@@ -326,15 +318,6 @@ class CEditDoc : public Manah::Windows::CDocument {
 
 	///	AhDAhDǗ
 	class CUndoManager /*: public Manah::CObject*/ {
-		// f[^o
-	private:
-		CEditDoc*					m_pDoc;			// ΏۃhLg
-		std::stack<COperationUnit*>	m_stkUndo;		// AhDX^bN
-		std::stack<COperationUnit*>	m_stkRedo;		// hDX^bN
-		bool						m_bVirtual;		// ẑƂ^
-		COperationUnit*				m_pVirtualUnit;	// zǉ鑀P
-		COperationUnit*				m_pLastUnit;	// ŌɒǉꂽP
-
 		// RXgN^
 	public:
 		CUndoManager(CEditDoc* pDoc);
@@ -342,13 +325,25 @@ class CEditDoc : public Manah::Windows::CDocument {
 
 		// \bh
 	public:
-		void	Clear();						// obt@ɂ
-		size_t	GetRedoBufferLength() const;	// hD\ȉ񐔂Ԃ
-		size_t	GetUndoBufferLength() const;	// AhD\ȉ񐔂Ԃ
-		void	PushUndoBuffer(					// AhDobt@ɑǉ
-					IOperation* pOperation, bool bConcatPrev);
-		void	Redo();							// hLgɃhDs
-		void	Undo();							// hLgɃAhDs
+		void		Clear();						// obt@ɂ
+		std::size_t	GetRedoBufferLength() const;	// hD\ȉ񐔂Ԃ
+		std::size_t	GetUndoBufferLength() const;	// AhD\ȉ񐔂Ԃ
+		bool		IsModifiedSinceLastSave() const;
+		void		OnSave();
+		void		PushUndoBuffer(					// AhDobt@ɑǉ
+						IOperation* pOperation, bool bConcatPrev);
+		void		Redo();							// hLgɃhDs
+		void		Undo();							// hLgɃAhDs
+
+		// f[^o
+	private:
+		CEditDoc*					m_pDoc;				// ΏۃhLg
+		std::stack<COperationUnit*>	m_stkUndo;			// AhDX^bN
+		std::stack<COperationUnit*>	m_stkRedo;			// hDX^bN
+		bool						m_bVirtual;			// ẑƂ^
+		COperationUnit*				m_pVirtualUnit;		// zǉ鑀P
+		COperationUnit*				m_pLastUnit;		// ŌɒǉꂽP
+		IOperation*					m_pSavedOperation;	// ۑɖɂȂĂ
 	};
 
 	///	AhDO[v̏
@@ -360,12 +355,12 @@ class CEditDoc : public Manah::Windows::CDocument {
 
 	// f[^o
 public:
-	static const wchar_t	m_wszBreakChars[6];		// s̏W
+	static const char_t	m_wszBreakChars[6];		// s̏W
 private:
 	bool						m_bIgnoreViews;	// OnUpdate ĂяoKvƂ true (t@CǂݍݓrȂ)
 	bool						m_bReadOnly;	// ǂݎp[h (ǂݍݎɐݒ肳s)
 	FileOpenMode				m_fomMode;		// t@C̋L[h (FOM_ASREADONLY ͖Ӗ)
-	UINT						m_nCodePage;	// R[hy[W
+	Manah::Text::CodePage		m_nCodePage;	// R[hy[W
 	BreakType					m_breakType;	// sR[h (L[{[h͂ɂsɎgp)
 	CEditDocLineList			m_listLines;	// s
 	Manah::Windows::IO::CFile	m_oFile;		// ݊JĂt@C
@@ -378,14 +373,14 @@ private:
 														// \bhĂяôƂ true B
 														// true ̊Ԃ͗\bhŃhDX^bNNAȂ
 
-	IEditDocEventListener*		m_pEventListener;	// CxgXi
+	std::set<IEditDocEventListener*>	m_setEventListeners;	// CxgXi
 	FILETIME					m_lastWriteTime;	// t@C̍ŏIXV
 	UINT						m_nTimerId;			// ^C} ID
 	static std::map<UINT, CEditDoc*>	m_mapDocuments;	// ^C} ID -> hLg̃}bv
 
-	mutable unsigned long						m_iLineCache_;	// GetInternalLineIterator LbVpf[^
+	mutable length_t							m_iLineCache_;	// GetInternalLineIterator LbVpf[^
 	mutable CEditDocLineList::iterator			m_itCache_;
-	mutable unsigned long						m_iLineCache;	// GetLineIterator LbVpf[^
+	mutable length_t							m_iLineCache;	// GetLineIterator LbVpf[^
 	mutable CEditDocLineList::const_iterator	m_itCache;
 
 	// RXgN^
@@ -395,83 +390,124 @@ public:
 
 	// \bh
 public:
+	// CxgXi
+	void	AddEventListener(IEditDocEventListener* pEventListener);
+	void	RemoveEventListener(IEditDocEventListener* pEventListener);
+
 	// hLgANZX
-	void				ClearUndoBuffer();												// AhD/hDX^bÑNA
-	void				GetAllLines(std::wstring& strText) const;						// Ss擾
-	CEditController*	GetController() const;											// Rg[Ԃ
-	unsigned long		GetDocumentLength() const;										// hLg̒Ԃ
-	const std::wstring&	GetLine(unsigned long iLine) const throw(out_of_range);			// 1s擾
-	unsigned long		GetLineCount() const;											// s擾
-	unsigned long		GetLineIndex(unsigned long iLine,								// s̃ItZbg擾
-							bool bIncludeBreak) const throw(out_of_range);
-	const CEditDocLine*	GetLineInfo(unsigned long iLine) const throw(out_of_range);		// s擾
-	LineIterator		GetLineIterator(unsigned long iLine) const throw(out_of_range);	// sCe[^̎擾
-	unsigned long		GetLineLength(unsigned long iLine) const throw(out_of_range);	// s̒擾
-	void				GetModification(TUpdateInfo& oUpdateInfo) const;				// XVe̎擾
-	FileOpenMode		GetShareMode() const;											// r[h̎擾
-	unsigned long		GetUndoHistoryLength(bool bRedo = false) const;					// AhD/hD\ȉ񐔂Ԃ
-	void				InitiateDocument();												// 
-	bool				IsReadOnly() const;												// ǂݎp[h̎擾
-	void				SetReadOnly(bool bReadOnly = true);								// ǂݎp[h̐ݒ
+	void				ClearUndoBuffer();
+	void				GetAllLines(std::wstring& strText) const;
+	CEditController*	GetController() const;
+	length_t			GetDocumentLength() const;
+	const string_t&		GetLine(length_t iLine) const throw(std::out_of_range);
+	length_t			GetLineCount() const;
+	length_t			GetLineIndex(length_t iLine,
+							bool bIncludeBreak) const throw(std::out_of_range);
+	const CEditDocLine*	GetLineInfo(length_t iLine) const throw(std::out_of_range);
+	LineIterator		GetLineIterator(length_t iLine) const throw(std::out_of_range);
+	length_t			GetLineLength(length_t iLine) const throw(std::out_of_range);
+	void				GetModification(TUpdateInfo& oUpdateInfo) const;
+	FileOpenMode		GetShareMode() const;
+	unsigned long		GetUndoHistoryLength(bool bRedo = false) const;
+	void				InitiateDocument();
+	bool				IsReadOnly() const;
+	void				SetReadOnly(bool bReadOnly = true);
 
 	// ҏW
 	void		BeginEditCollection();
-	CCharPos	DeleteText(Manah::CObject* pSender,							// w͈͂폜
+	CCharPos	DeleteText(Manah::CObject* pSender,
 					const CCharPos& posBegin,
 					const CCharPos& posEnd) throw(EDocumentIsReadOnly);
 	void		EndEditCollection();
-	CCharPos	InsertText(Manah::CObject* pSender,							// wʒuɕ}
+	CCharPos	InsertText(Manah::CObject* pSender,
 					const CCharPos& pos,
-					const std::wstring& strText) throw(EDocumentIsReadOnly);
-	void		Redo() throw(EDocumentIsReadOnly);							// hD̎s
-	void		SetEventListener(IEditDocEventListener* pEventListener);	// CxgXi̐ݒ
-	void		Undo() throw(EDocumentIsReadOnly);							// AhD̎s
+					const string_t& strText) throw(EDocumentIsReadOnly);
+	bool		IsCollectingEdit() const;
+	void		Redo() throw(EDocumentIsReadOnly);
+	void		Undo() throw(EDocumentIsReadOnly);
 
 	// R[h
-	static wstring	GetBreakString(BreakType bt);
-	BreakType		GetBreakType() const;									// ݂̉sR[hԂ
-	UINT			GetCodePage() const;									// ݂̃R[hy[WԂ
-	static bool		IsAvailableCodePage(UINT nCodePage);					// ̃NXŗp\ȃR[hy[Wǂ
-	bool			IsCharCodeUnicode() const;								// gp̕R[h Unicode 
-	void			SetBreakType(BreakType bt) throw(invalid_argument);		// sR[h̐ݒ
-	void			SetCodePage(UINT nCodePage) throw(invalid_argument);	// R[hy[W̐ݒ
+	static string_t			GetBreakString(BreakType bt);
+	BreakType				GetBreakType() const;
+	Manah::Text::CodePage	GetCodePage() const;
+	void					SetBreakType(BreakType bt) throw(std::invalid_argument);
+	void					SetCodePage(Manah::Text::CodePage cp) throw(std::invalid_argument);
 
 	// t@C <-> hLg
-	// 3̃\bh͉炩̗RŎsꍇ́AG[̓e\_CAO\A
-	// false ܂ StreamStatus ԂB͂̃\bhŐZ
-	bool			CloseDocument();										// t@C
-	StreamStatus	LoadDocument(const std::wstring& strPathName,			// t@C̓ǂݍ
-						FileOpenMode fom, UINT nCodePage = EXTCP_AUTO);
-	StreamStatus	SaveDocument(const std::wstring& strPathName,			// t@C̕ۑ
-						SaveDocumentOption sdo,
-						BreakType bt, UINT nCodePage = EXTCP_AUTO);
+	bool			CloseDocument();
+	StreamStatus	LoadDocument(const std::wstring& strPathName,
+						FileOpenMode fom, Manah::Text::CodePage cp = Manah::Text::CPEX_AUTODETECT_USERLANG);
+	StreamStatus	SaveDocument(const std::wstring& strPathName, SaveDocumentOption sdo,
+						BreakType bt, Manah::Text::CodePage cp = Manah::Text::CPEX_AUTODETECT);
 
 	// ɊJĂt@Cɑ΂鏈
-	OperationStatus	CopyCurrentFile(const wchar_t* pwszDestination);				// Rs[
-	OperationStatus	DeleteCurrentFile() throw(EDocumentIsReadOnly);					// ݔֈړ
-	bool			LockCurrentFile(FileOpenMode fom);								// bN
-	OperationStatus	MoveCurrentFile(const wchar_t* pwszDestination);				// ړ
-	bool			SendCurrentFile(bool bAsAttachment, bool bShowDialog = true);	// [֑M
+	OperationStatus	CopyCurrentFile(const wchar_t* pwszDestination);
+	OperationStatus	DeleteCurrentFile() throw(EDocumentIsReadOnly);
+	bool			LockCurrentFile(FileOpenMode fom);
+	OperationStatus	MoveCurrentFile(const wchar_t* pwszDestination);
+	bool			SendCurrentFile(bool bAsAttachment, bool bShowDialog = true);
 
 protected:
 	CEditDocLineList::iterator
-		GetInternalLineIterator(unsigned long iLine) const throw(out_of_range);	// const łȂsCe[^̎擾
+		GetInternalLineIterator(length_t iLine) const throw(std::out_of_range);
 	void	CheckFileLastWriteTime();
 private:
-	static void CALLBACK	TimerProc(HWND hWnd, UINT nMsg, UINT_PTR idEvent, DWORD dwTime);
+	static void CALLBACK	TimerProc(HWND hWnd, UINT nMsg, TIMERPROC_3RD_PARAM_TYPE idEvent, DWORD dwTime);
 
 public:
-	virtual bool	OnNewDocument();									// VhLg쐬ƂɌĂяo
-	virtual bool	OnOpenDocument(const std::wstring& strPathName);	// t@CJƂɌĂяo
-	virtual bool	OnSaveDocument(const std::wstring& strPathName);	// t@CۑƂɌĂяo
+	virtual bool	OnNewDocument();
+	virtual bool	OnOpenDocument(const std::wstring& strPathName);
+	virtual bool	OnSaveDocument(const std::wstring& strPathName);
 };
 
 
 // CEditDoc class inline implementation
 /////////////////////////////////////////////////////////////////////////////
 
+/**
+ *	CxgXi̒ǉ
+ *	@param pEventListener	VCxgXi
+ */
+inline void CEditDoc::AddEventListener(IEditDocEventListener* pEventListener) {
+	AssertValid();
+	assert(pEventListener != 0);
+	m_setEventListeners.insert(pEventListener);
+}
+
+///	AhDO[v̎WJnB
+///	ݎWł΁AxIAVWJn
+inline void CEditDoc::BeginEditCollection() {
+	AssertValid();
+
+	if(m_ugsGroupingState != UGS_NONE)
+		EndEditCollection();
+	m_ugsGroupingState = UGS_WAITFORFIRSTEDIT;
+}
+
+///	AhD/hDX^bNɂė
+inline void CEditDoc::ClearUndoBuffer() {
+	AssertValid();
+	m_pUndoManager->Clear();
+	m_bOnceUndoBufferCleared = true;
+}
+
+///	hLg OnCloseDocument Ă
+inline bool CEditDoc::CloseDocument() {
+	AssertValid();
+	m_oFile.Close();
+	InitiateDocument();
+	OnCloseDocument();
+	return true;
+}
+
+///	AhDO[v̎WI
+inline void CEditDoc::EndEditCollection() {
+	AssertValid();
+	m_ugsGroupingState = UGS_NONE;
+}
+
 ///	sR[h̉sԂ
-inline wstring CEditDoc::GetBreakString(BreakType bt) {
+inline string_t CEditDoc::GetBreakString(BreakType bt) {
 	switch(bt) {
 	case BT_AUTO:	return L"";
 	case BT_LF:		return L"\n";
@@ -503,12 +539,46 @@ inline CEditController* CEditDoc::GetController() const {
 	return reinterpret_cast<CEditController*>(m_pController);
 }
 
+/**
+ *	w肵s̃eLXg擾B
+ *	s݂Ȃ <code>out_of_range</code> 𓊂
+ */
+inline const wstring& CEditDoc::GetLine(length_t iLine) const throw(std::out_of_range) {
+	AssertValid();
+	try {
+		return GetLineInfo(iLine)->m_strLine;
+	} catch(std::out_of_range& e) {
+		throw e;
+	}
+}
+
 ///	s擾
-inline unsigned long CEditDoc::GetLineCount() const {
+inline length_t CEditDoc::GetLineCount() const {
 	AssertValid();
 	return m_listLines.size();
 }
 
+///	sꊇĎ擾
+inline const CEditDocLine* CEditDoc::GetLineInfo(length_t iLine) const throw(std::out_of_range) {
+	AssertValid();
+
+	try {
+		return &(*GetLineIterator(iLine));
+	} catch(std::out_of_range& e) {
+		throw e;
+	}
+}
+
+///	w肵s̒擾Bs݂Ȃ <code>out_of_range</code> 𓊂
+inline length_t CEditDoc::GetLineLength(length_t iLine) const throw(std::out_of_range) {
+	AssertValid();
+	try {
+		return GetLine(iLine).length();
+	} catch(std::out_of_range& e) {
+		throw e;
+	}
+}
+
 ///	Ō̍XV擾
 inline void CEditDoc::GetModification(TUpdateInfo& oUpdateInfo) const {
 	AssertValid();
@@ -524,31 +594,14 @@ inline FileOpenMode CEditDoc::GetShareMode() const {
 ///	AhDAhD\ȉ񐔂擾
 inline unsigned long CEditDoc::GetUndoHistoryLength(bool bRedo /* = false */) const {
 	AssertValid();
-	return bRedo ? m_pUndoManager->GetRedoBufferLength() : m_pUndoManager->GetUndoBufferLength();
+	return bRedo ? m_pUndoManager->GetRedoBufferLength()
+		: m_pUndoManager->GetUndoBufferLength();
 }
 
-///	̃NXŗp\ȃR[hy[WǂԂ (EXTCP_AUTO ɂ false Ԃ)
-inline bool CEditDoc::IsAvailableCodePage(UINT nCodePage) {
-	return nCodePage == CP_UTF8
-		|| nCodePage == EXTCP_EUCJP
-		|| nCodePage == EXTCP_UTF16LE
-		|| nCodePage == EXTCP_UTF16BE
-		|| nCodePage == EXTCP_UTF32LE
-		|| nCodePage == EXTCP_UTF32BE
-		|| nCodePage == EXTCP_UTF5
-		|| ::IsValidCodePage(nCodePage);
-}
-
-///	ݎgpĂR[hy[W Unicode Ԃ
-inline bool CEditDoc::IsCharCodeUnicode() const {
+///	AhDO[v̎WԂ
+inline bool CEditDoc::IsCollectingEdit() const {
 	AssertValid();
-	return m_nCodePage == EXTCP_UTF5
-		|| m_nCodePage == CP_UTF7
-		|| m_nCodePage == CP_UTF8
-		|| m_nCodePage == EXTCP_UTF16LE
-		|| m_nCodePage == EXTCP_UTF16BE
-		|| m_nCodePage == EXTCP_UTF16LE
-		|| m_nCodePage == EXTCP_UTF32BE;
+	return m_ugsGroupingState != UGS_NONE;
 }
 
 ///	ǂݎp[hǂԂ
@@ -557,6 +610,71 @@ inline bool CEditDoc::IsReadOnly() const {
 	return m_bReadOnly;
 }
 
+/**
+ *	CxgXi̍폜
+ *	@param pEventListener	폜CxgXi
+ */
+inline void CEditDoc::RemoveEventListener(IEditDocEventListener* pEventListener) {
+	AssertValid();
+	assert(pEventListener != 0);
+	m_setEventListeners.erase(pEventListener);
+}
+
+/**
+ *	sR[h̐ݒ
+ *	@param breakType		Vݒ肷sR[h
+ *	@throw invalid_argument	sR[hȂƂX[
+ */
+inline void CEditDoc::SetBreakType(BreakType breakType) throw(std::invalid_argument) {
+	AssertValid();
+	switch(breakType) {
+	case BT_LF:		case BT_CR:
+	case BT_CRLF:	case BT_NEL:
+	case BT_LS:		case BT_PS:
+		m_breakType = breakType;
+		break;
+	default:
+		throw std::invalid_argument("Specified break type is invalid.");
+	}
+}
+
+/**
+ *	R[hy[W̐ݒ
+ *	@param cp				Vݒ肷R[hy[Wԍ
+ *	@throw invalid_argument	R[hy[WpłȂƂX[
+ */
+inline void CEditDoc::SetCodePage(Manah::Text::CodePage cp) throw(std::invalid_argument) {
+	AssertValid();
+	if(!Manah::Text::CEncoder::IsValidCodePage(cp)
+			|| Manah::Text::CEncoder::IsCodePageForAutoDetection(cp))
+		throw std::invalid_argument("Specified code page is not available.");
+	m_nCodePage = cp;
+}
+
+///	ǂݎp[h̐ݒ
+inline void CEditDoc::SetReadOnly(bool bReadOnly /* = true */) {
+	AssertValid();
+	m_bReadOnly = bReadOnly;
+	for(set<IEditDocEventListener*>::iterator it
+			= m_setEventListeners.begin(); it != m_setEventListeners.end(); ++it)
+		(*it)->OnDocumentModified();
+}
+
+///	hLgV쐬ƂɌĂяo
+inline bool CEditDoc::OnNewDocument() {
+	return true;
+}
+
+///	hLgǂݍ݂̂߂Ƀt@CJƂɌĂяo
+inline bool CEditDoc::OnOpenDocument(const std::wstring& strPathName) {
+	return true;
+}
+
+///	hLgۑƂɌĂяo
+inline bool CEditDoc::OnSaveDocument(const std::wstring& strPathName) {
+	return true;
+}
+
 } // namespace Ascension
 
 
