@@ -9,6 +9,7 @@
 using namespace Ascension;
 using namespace Manah;
 using namespace Manah::Windows;
+using namespace Manah::Text;
 using namespace std;
 
 
@@ -21,7 +22,8 @@ using namespace std;
  */
 CEditPoint::CEditPoint(CEditView* pView)
 		: m_iLine(0), m_iChar(0), m_pView(pView),
-		m_bSyncWithOtherEdit(false), m_bOwnedByView(false), m_cxLast(0) {
+		m_bSyncWithOtherEdit(false), m_bOwnedByView(false), 
+		m_ubBehavior(UB_CLUSTER), m_cxLast(0) {
 	assert(m_pView != 0);
 }
 
@@ -33,54 +35,31 @@ CEditPoint::~CEditPoint() {
 
 /**
  *	̕Ɉړ
- *	@param nOffset	ړ邩
+ *	@param cch	ړ邩
  */
-void CEditPoint::CharNext(unsigned long nOffset /* = 1 */) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::CharNext(length_t cch /* = 1 */) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
-	unsigned long	cLines = m_pView->GetDocument()->GetLineCount();
-	wstring			strLine = m_pView->GetDocument()->GetLine(m_iLine);
-	const wchar_t*	pwszLine = strLine.c_str();
-
 	Normalize();
-	while(nOffset-- > 0) {
-		if(m_iChar == strLine.length()) {	// sȂ̂Ŏ̍sɈړ
-			if(m_iLine == cLines - 1)	// ŏIsłΈړȂ
-				return;
-			strLine = m_pView->GetDocument()->GetLine(++m_iLine);
-			pwszLine = strLine.c_str();
-			m_iChar = 0;
-		} else
-			// Lbg󂯎Ȃ΂
-			m_iChar += CEditView::MakeCaretPosValid(pwszLine, strLine.length(), ++m_iChar, false);
-	}
+	const CCharPos	pos = GetNextCharPos(*this, cch, m_ubBehavior);
+	m_iLine = pos.m_iLine;
+	m_iChar = pos.m_iChar;
 	UpdateLastCx();
 }
 
 /**
  *	O̕Ɉړ
- *	@param nOffset	ړ邩
+ *	@param cch	ړ邩
  */
-void CEditPoint::CharPrev(unsigned long nOffset /* = 1 */) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::CharPrev(length_t cch /* = 1 */) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
-	wstring			strLine = m_pView->GetDocument()->GetLine(m_iLine);
-	const wchar_t*	pwszLine = strLine.c_str();
-
 	Normalize();
-	while(nOffset-- > 0) {
-		if(m_iChar == 0) {	// sȂ̂őO̍sɈړ
-			if(m_iLine == 0)	// 擪słΈړȂ
-				return;
-			strLine = m_pView->GetDocument()->GetLine(--m_iLine);
-			pwszLine = strLine.c_str();
-			m_iChar = strLine.length();
-		} else
-			// Lbg󂯎Ȃ΂
-			m_iChar += CEditView::MakeCaretPosValid(pwszLine, strLine.length(), --m_iChar, true);
-	}
+	const CCharPos	pos = GetPrevCharPos(*this, cch, m_ubBehavior);
+	m_iLine = pos.m_iLine;
+	m_iChar = pos.m_iChar;
 	UpdateLastCx();
 }
 
@@ -102,7 +81,7 @@ CEditPoint* CEditPoint::Clone() const throw(EViewIsAlreadyInavailable) {
  *	@param cChars	1̈ʒu܂ł̕ (ł悢)
  *	@param rct		ϊ̎
  */
-void CEditPoint::Convert(long cChars, RangeConvertType rct) {
+void CEditPoint::Convert(signed_length_t cChars, RangeConvertType rct) {
 	AssertValid();
 	VERIFY_VIEW();
 
@@ -149,7 +128,7 @@ void CEditPoint::Convert(const CCharPos& pos, RangeConvertType rct) {
  *	wʒu܂ł̃eLXgNbv{[hɃRs[
  *	@param cChars	Rs[镶 (ł悢)
  */
-void CEditPoint::Copy(long cChars) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::Copy(signed_length_t cChars) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 	CClipboard::SetClipboardText(m_pView->m_hWnd, GetText(cChars).c_str());
@@ -169,7 +148,7 @@ void CEditPoint::Copy(const CCharPos& pos) throw(EViewIsAlreadyInavailable) {
  *	wʒu܂ł̃eLXg폜ăNbv{[hɃRs[
  *	@param cChars	폜镶 (ł悢)
  */
-void CEditPoint::Cut(long cChars) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::Cut(signed_length_t cChars) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
@@ -196,8 +175,9 @@ void CEditPoint::Cut(const CCharPos& pos) throw(EViewIsAlreadyInavailable) {
 /**
  *	wʒu܂ł̃eLXg폜
  *	@param cChars	1̈ʒu܂ł̕ (ł悢)
+ *	@param ub		̌vZ@BUB_DEFAULT w肷ƌ݂̐ݒ肪gp
  */
-void CEditPoint::Delete(long cChars /* = 1 */) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::Delete(signed_length_t cChars /* = 1 */, UnitBehavior ub /* = UB_DEFAULT */) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
@@ -241,61 +221,97 @@ void CEditPoint::Delete(const CCharPos& pos) throw(EViewIsAlreadyInavailable) {
 	UpdateLastCx();
 }
 
-///	_̍sr[̒ɗ悤ɃXN[
-void CEditPoint::EnsureCentered() throw(EViewIsAlreadyInavailable) {
+/**
+ *	w͈͂r[̒ɂȂ悤ɃXN[BɉȂ牽Ȃ
+ *	@param cChars	͈͂\̓_܂ł̕
+ *	@return			͈͂r[ɔ[܂ꍇ true Ԃ
+ */
+bool CEditPoint::EnsureCentered(signed_length_t cChars /* = 0 */) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
-
-	const CCharPos		posDisplay = m_pView->DisplayCharFromLogicalChar(*this);
-	const unsigned int	cVisibleLines = m_pView->GetVisibleLineCount();
-
-	m_pView->OnHScroll(SB_THUMBPOSITION, 0, 0);
-	m_pView->OnVScroll(SB_THUMBPOSITION, posDisplay.m_iLine - cVisibleLines / 2, 0);
+	return EnsureCentered((cChars >= 0) ?
+		GetNextCharPos(*this, cChars, m_ubBehavior) :
+		GetPrevCharPos(*this, -cChars, m_ubBehavior));
 }
 
-///	_ɂȂ悤Ƀr[XN[
-void CEditPoint::EnsureVisible() throw(EViewIsAlreadyInavailable) {
+/**
+ *	w͈͂r[̒ɂȂ悤ɃXN[BɉȂ牽Ȃ
+ *	@param pos	͈͂\̓_
+ *	@return		͈͂r[ɔ[܂ꍇ true Ԃ (ɂ true)
+ */
+bool CEditPoint::EnsureCentered(const CCharPos& pos) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
-	const CCharPos		posDisplay = m_pView->DisplayCharFromLogicalChar(*this);
-	const unsigned int	cVisibleLines = m_pView->GetVisibleLineCount();
-	const unsigned int	cVisibleChars = m_pView->GetVisibleCharCount();
+	const CCharPos	posDisplay = m_pView->DisplayCharFromLogicalChar(*this);
+	const length_t	cVisibleLines = m_pView->GetVisibleLineCount();
+
+	m_pView->OnHScroll(SB_LEFT, 0, 0);
+	m_pView->OnVScroll(SB_SETPOS, posDisplay.m_iLine - cVisibleLines / 2, 0);
+
+	return true;
+}
+
+/**
+ *	w͈͂ɂȂ悤Ƀr[XN[
+ *	@param cChars	͈͂\̓_܂ł̕
+ *	@return			͈͂r[ɔ[܂ꍇ true Ԃ (ɂ true)
+ */
+bool CEditPoint::EnsureVisible(signed_length_t cChars /* = 0 */) throw(EViewIsAlreadyInavailable) {
+	AssertValid();
+	VERIFY_VIEW();
+	return EnsureVisible((cChars >= 0) ?
+		GetNextCharPos(*this, cChars, m_ubBehavior) :
+		GetPrevCharPos(*this, -cChars, m_ubBehavior));
+}
+
+/**
+ *	_ɂȂ悤Ƀr[XN[
+ *	@param	pos	͈͂\̓_
+ *	@return		͈͂r[ɔ[܂ꍇ true Ԃ (ɂ true)
+ */
+bool CEditPoint::EnsureVisible(const CCharPos& pos) throw(EViewIsAlreadyInavailable) {
+	AssertValid();
+	VERIFY_VIEW();
+
+	const CCharPos	posDisplay = m_pView->DisplayCharFromLogicalChar(*this);
+	const length_t	cVisibleLines = m_pView->GetVisibleLineCount();
+	const length_t	cVisibleChars = m_pView->GetVisibleCharCount();
 
 	if(posDisplay.m_iLine <
 			m_pView->m_ptScroll.y * m_pView->m_layoutInfo.nVScrollRatio)	// ʂ
-		m_pView->OnVScroll(SB_THUMBPOSITION, posDisplay.m_iLine, 0);
+		m_pView->OnVScroll(SB_SETPOS, posDisplay.m_iLine, 0);
 	else if(posDisplay.m_iLine -
 			m_pView->m_ptScroll.y *
 			m_pView->m_layoutInfo.nVScrollRatio > cVisibleLines - 1)	// ʂ艺
-		m_pView->OnVScroll(SB_THUMBPOSITION, posDisplay.m_iLine - cVisibleLines + 1, 0);
-	if(m_pView->m_ModeState.wpmWrapMode == WPM_NONE) {
+		m_pView->OnVScroll(SB_SETPOS, posDisplay.m_iLine - cVisibleLines + 1, 0);
+	if(m_pView->m_modeState.wpmWrapMode == WPM_NONE) {
 		const unsigned long	nScrollOffset =
 			m_pView->m_ptScroll.x * m_pView->m_layoutInfo.nHScrollRatio * m_pView->GetAvgCharWidth();
 		const unsigned long	cx =
 			m_pView->PosFromChar(*this).x + nScrollOffset - m_pView->m_layoutInfo.nLeftTabWidth + 1;
 		if(cx <= nScrollOffset)	// ʂ荶
-			m_pView->OnHScroll(SB_THUMBPOSITION,
-				cx / m_pView->GetAvgCharWidth() - cVisibleChars / 4, 0);
+			m_pView->OnHScroll(SB_SETPOS, cx / m_pView->GetAvgCharWidth() - cVisibleChars / 4, 0);
 		else if(cx > (m_pView->m_ptScroll.x * m_pView->m_layoutInfo.nHScrollRatio
 				+ cVisibleChars) * m_pView->GetAvgCharWidth())	// ʂE
-			m_pView->OnHScroll(SB_THUMBPOSITION,
-				cx / m_pView->GetAvgCharWidth() - cVisibleChars * 3 / 4, 0);
+			m_pView->OnHScroll(SB_SETPOS, cx / m_pView->GetAvgCharWidth() - cVisibleChars * 3 / 4, 0);
 	} else {
 		// ...
 	}
+
+	return true;
 }
 
 ///	hLg̐擪̕Ԃ
-unsigned long CEditPoint::GetAbsoluteCharOffset() const throw(EViewIsAlreadyInavailable) {
+length_t CEditPoint::GetAbsoluteCharOffset() const throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
-	unsigned long	nOffset = 0;
+	length_t		nOffset = 0;
 	LineIterator	itLines = m_pView->GetDocument()->GetLineIterator(0);
 
 	Normalize();
-	for(unsigned long iLine = 0; ; ++iLine, ++itLines) {
+	for(length_t iLine = 0; ; ++iLine, ++itLines) {
 		if(iLine == m_iLine) {
 			nOffset += m_iChar;
 			break;
@@ -306,21 +322,34 @@ unsigned long CEditPoint::GetAbsoluteCharOffset() const throw(EViewIsAlreadyInav
 }
 
 ///	ԍԂ
-unsigned long CEditPoint::GetCharNumber() const throw(EViewIsAlreadyInavailable) {
+length_t CEditPoint::GetCharNumber() const throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 	return m_iChar;
 }
 
+///	݈ʒũR[h|CgԂBs̏ꍇ͎ۂ̉sR[hɊ֌W 0x000A A
+///	̖̏ꍇ 0xFFFFFFFF Ԃ
+CodePoint CEditPoint::GetCodePoint() const throw(EViewIsAlreadyInavailable) {
+	AssertValid();
+	VERIFY_VIEW();
+
+	const string_t&	strLine = m_pView->GetDocument()->GetLine(m_iLine);
+
+	if(m_iChar == strLine.length())
+		return (m_iLine == m_pView->GetDocument()->GetLineCount() - 1) ? 0xFFFFFFFF : 0x000A;
+	return DecodeUTF16SurrogatePairToCodePoint(strLine.c_str() + m_iChar, strLine.length() - m_iChar);
+}
+
 ///	\̗ԍԂ
-unsigned long CEditPoint::GetColumnNumber() const throw(EViewIsAlreadyInavailable) {
+length_t CEditPoint::GetColumnNumber() const throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 	return m_pView->ColumnFromChar(*this);
 }
 
 ///	_s̒Ԃ
-unsigned long CEditPoint::GetLineLength() const throw(EViewIsAlreadyInavailable) {
+length_t CEditPoint::GetLineLength() const throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 	Normalize();
@@ -328,30 +357,109 @@ unsigned long CEditPoint::GetLineLength() const throw(EViewIsAlreadyInavailable)
 }
 
 ///	sԍԂ
-unsigned long CEditPoint::GetLineNumber() const throw(EViewIsAlreadyInavailable) {
+length_t CEditPoint::GetLineNumber() const throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 	Normalize();
 	return m_iLine;
 }
 
+/**
+ *	^ꂽʒuw蕶i񂾈ʒuԂ
+ *	@param pt	ʒu
+ *	@param cch	
+ *	@param ub	vZ@
+ */
+CCharPos CEditPoint::GetNextCharPos(const CEditPoint& pt, length_t cch, UnitBehavior ub) {
+	length_t		cLines = pt.m_pView->GetDocument()->GetLineCount();
+	string_t		strLine = pt.m_pView->GetDocument()->GetLine(pt.m_iLine);
+	const char_t*	pwszLine = strLine.c_str();
+	CCharPos		pos = pt;
+
+	while(cch-- > 0) {
+		if(pos.m_iChar == strLine.length()) {	// sȂ̂Ŏ̍sɈړ
+			if(pos.m_iLine == cLines - 1)	// ŏIsłΈړȂ
+				return pos;
+			strLine = pt.m_pView->GetDocument()->GetLine(++pos.m_iLine);
+			pwszLine = strLine.c_str();
+			pos.m_iChar = 0;
+		} else if(ub == UB_UTF16)
+			++pos.m_iChar;
+		else {
+			CodePoint	cp;
+			cp = DecodeUTF16SurrogatePairToCodePoint(pwszLine + pos.m_iChar, strLine.length() - pos.m_iChar);
+			pos.m_iChar += (cp > 0xFFFF) ? 2 : 1;
+			while(ub == UB_CLUSTER && pos.m_iChar < strLine.length()) {
+				cp = DecodeUTF16SurrogatePairToCodePoint(pwszLine + pos.m_iChar, strLine.length() - pos.m_iChar);
+				if(CBoundarySearcher::IsFirstCharacterOfCluster(cp))
+					break;
+				pos.m_iChar += (cp > 0xFFFF) ? 2 : 1;
+			}
+		}
+	}
+	return pos;
+}
+
+/**
+ *	^ꂽʒuw蕶߂ʒuԂ
+ *	@param pt	ʒu
+ *	@param cch	
+ *	@param ub	vZ@
+ */
+CCharPos CEditPoint::GetPrevCharPos(const CEditPoint& pt, length_t cch, UnitBehavior ub) {
+	string_t		strLine = pt.m_pView->GetDocument()->GetLine(pt.m_iLine);
+	const char_t*	pwszLine = strLine.c_str();
+	CCharPos		pos = pt;
+
+	while(cch-- > 0) {
+		if(pos.m_iChar == 0) {	// sȂ̂őO̍sɈړ
+			if(pos.m_iLine == 0)	// 擪słΈړȂ
+				return pos;
+			strLine = pt.m_pView->GetDocument()->GetLine(--pos.m_iLine);
+			pwszLine = strLine.c_str();
+			pos.m_iChar = strLine.length();
+		} else if(ub == UB_UTF16 || pos.m_iChar == 0)
+			--pos.m_iChar;
+		else {
+			CodePoint	cp;
+			if(IsUTF16HighSurrogate(pwszLine[pos.m_iChar - 2])
+					&& IsUTF16LowSurrogate(pwszLine[pos.m_iChar - 1]))
+				cp = DecodeUTF16SurrogatePairToCodePoint(pwszLine + pos.m_iChar - 2, 2);
+			else
+				cp = pwszLine[pos.m_iChar - 1];
+			pos.m_iChar -= (cp > 0xFFFF) ? 2 : 1;
+			while(ub == UB_CLUSTER
+					&& pos.m_iChar != 0
+					&& !CBoundarySearcher::IsFirstCharacterOfCluster(cp)) {
+				if(pos.m_iChar != 1
+						&& IsUTF16HighSurrogate(pwszLine[pos.m_iChar - 2])
+						&& IsUTF16LowSurrogate(pwszLine[pos.m_iChar - 1])) {
+					cp = DecodeUTF16SurrogatePairToCodePoint(pwszLine + pos.m_iChar - 2, 2);
+					pos.m_iChar -= 2;
+				} else {
+					cp = pwszLine[pos.m_iChar - 1];
+					--pos.m_iChar;
+				}
+			}
+		}
+	}
+	return pos;
+}
+
 /**
  *	͈͓̃eLXgԂ
  *	@param cChars	1̈ʒu܂ł̕ (ł悢)
  */
-wstring CEditPoint::GetText(long cChars) const throw(EViewIsAlreadyInavailable) {
+string_t CEditPoint::GetText(signed_length_t cChars) const throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
 	Normalize();
 	if(cChars == 0)
 		return L"";
-
-	const CEditDoc*	pDoc = m_pView->GetDocument();
-	wostringstream	ssText;
-
-	while(true) {
-	}
+	return GetText((cChars > 0) ?
+		GetNextCharPos(*this, cChars, m_ubBehavior) :
+		GetPrevCharPos(*this, -cChars, m_ubBehavior));
 }
 
 /**
@@ -362,29 +470,35 @@ wstring CEditPoint::GetText(const CCharPos& pos) const throw(EViewIsAlreadyInava
 	AssertValid();
 	VERIFY_VIEW();
 
+	CCharPos	pos_ = pos;
+	
+	pos_.m_iLine = std::min(pos.m_iLine, m_pView->GetDocument()->GetLineCount() - 1);
 	Normalize();
 	if(*this == pos)
 		return L"";
 
 	const CEditDoc*	pDoc = m_pView->GetDocument();
-	const CCharPos&	posStart = std::min<CCharPos>(*this, pos);
-	const CCharPos&	posEnd = std::max<CCharPos>(*this, pos);
+	const CCharPos&	posStart = std::min<CCharPos>(*this, pos_);
+	const CCharPos&	posEnd = std::max<CCharPos>(*this, pos_);
 
 	if(posStart.m_iLine == posEnd.m_iLine)	// 1s̏ꍇ
 		return pDoc->GetLine(posEnd.m_iLine).substr(
 			posStart.m_iChar, posEnd.m_iChar - posStart.m_iChar);
 	else {	// s̏ꍇ
-		wostringstream	ssText;
-		unsigned long	iLine = posStart.m_iLine;
+		ostringstream_t	ssText;
+		length_t		iLine = posStart.m_iLine;
 		LineIterator	itLines = pDoc->GetLineIterator(iLine);
 
 		while(true) {
 			if(iLine == posStart.m_iLine)	// 擪s
-				ssText << itLines->GetLine().substr(posStart.m_iChar);
-			else if(iLine == posEnd.m_iLine)	// ŏIs
+				ssText << itLines->GetLine().substr(posStart.m_iChar)
+						<< CEditDoc::GetBreakString(itLines->GetBreakType());
+			else if(iLine == posEnd.m_iLine) {	// ŏIs
 				ssText << itLines->GetLine().substr(0, posEnd.m_iChar);
-			else
-				ssText << itLines->GetLine() << CEditDoc::GetBreakString(itLines->GetBreakType());
+				break;
+			} else
+				ssText << itLines->GetLine()
+						<< CEditDoc::GetBreakString(itLines->GetBreakType());
 			++iLine;
 			++itLines;
 		}
@@ -397,7 +511,7 @@ wstring CEditPoint::GetText(const CCharPos& pos) const throw(EViewIsAlreadyInava
  *	@param strText	}eLXg
  *	@param ocf		쌋tO
  */
-void CEditPoint::DestructiveInsert(const wstring& strText) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::DestructiveInsert(const string_t& strText) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
@@ -425,7 +539,7 @@ void CEditPoint::DestructiveInsert(const wstring& strText) throw(EViewIsAlreadyI
  *	@param chIndent	CfgɎg
  *	@param nLevel	Cfgx
  */
-CCharPos CEditPoint::Indent(const CCharPos& pos, wchar_t chIndent, long nLevel) {
+CCharPos CEditPoint::Indent(const CCharPos& pos, char_t chIndent, long nLevel) {
 	AssertValid();
 	VERIFY_VIEW();
 
@@ -433,12 +547,13 @@ CCharPos CEditPoint::Indent(const CCharPos& pos, wchar_t chIndent, long nLevel)
 		return pos;
 
 	CEditDoc*		pDocument = m_pView->GetDocument();
-	const wstring	strIndent = wstring(abs(nLevel), chIndent);
+	const CLexer*	pLexer = m_pView->GetLexer();
+	const string_t	strIndent = wstring(abs(nLevel), chIndent);
 	const CCharPos	posTop = m_pView->GetSelTopPoint();
 	const CCharPos	posBottom = m_pView->GetSelBottomPoint();
 	CCharPos		posOtherRes = pos;
-	unsigned long	iLine = posTop.m_iLine;
-	unsigned long	cchIndent;	// s̋󔒕
+	length_t		iLine = posTop.m_iLine;
+	length_t		cchIndent;	// s̋󔒕
 
 	if(iLine < posBottom.m_iLine && nLevel != 0) {
 		if(nLevel > 0) {
@@ -448,11 +563,11 @@ CCharPos CEditPoint::Indent(const CCharPos& pos, wchar_t chIndent, long nLevel)
 			else if(iLine == m_iLine && m_iChar != 0)
 				m_iChar += nLevel;
 		} else {
-			cchIndent = CEditView::IsWhiteSpace(
+			cchIndent = pLexer->IsWhiteSpace(
 							pDocument->GetLine(iLine).c_str(),
-							pDocument->GetLineLength(iLine));
+							pDocument->GetLineLength(iLine), true);
 			if(cchIndent > 0) {
-				unsigned long	cchDelete = min<unsigned long>(-nLevel, cchIndent);
+				const length_t	cchDelete = min<length_t>(-nLevel, cchIndent);
 				pDocument->DeleteText(this, CCharPos(iLine, 0), CCharPos(iLine, cchDelete));
 				if(iLine == posOtherRes.m_iLine && posOtherRes.m_iChar != 0)
 					posOtherRes.m_iChar -= cchDelete;
@@ -479,11 +594,11 @@ CCharPos CEditPoint::Indent(const CCharPos& pos, wchar_t chIndent, long nLevel)
 		}
 	} else {
 		for(++iLine; iLine <= posBottom.m_iLine; ++iLine) {
-			cchIndent = CEditView::IsWhiteSpace(
+			cchIndent = pLexer->IsWhiteSpace(
 							pDocument->GetLine(iLine).c_str(),
-							pDocument->GetLineLength(iLine));
+							pDocument->GetLineLength(iLine), true);
 			if(cchIndent > 0) {
-				unsigned long	cchDelete = min<unsigned long>(-nLevel, cchIndent);
+				const length_t	cchDelete = min<length_t>(-nLevel, cchIndent);
 				pDocument->DeleteText(this, CCharPos(iLine, 0), CCharPos(iLine, cchDelete));
 				if(iLine == posOtherRes.m_iLine && posOtherRes.m_iChar != 0)
 					posOtherRes.m_iChar -= cchDelete;
@@ -513,23 +628,23 @@ void CEditPoint::Insert(const wstring& strText) throw(EViewIsAlreadyInavailable)
  *	_̈ʒuɕ`}B݈ʒu͓͕i߂
  *	@param strText	}eLXg
  */
-void CEditPoint::InsertBox(const wstring& strText) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::InsertBox(const string_t& strText) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
 	if(m_pView->GetDocument()->IsReadOnly() || strText.empty())
 		return;
 
-	CEditDoc*			pDocument = m_pView->GetDocument();
-	const unsigned long	cLines = pDocument->GetLineCount();
-	const wstring		pwszBreaks = CEditDoc::GetBreakString(pDocument->GetBreakType());
-	list<size_t>		listLines;
+	CEditDoc*		pDocument = m_pView->GetDocument();
+	const length_t	cLines = pDocument->GetLineCount();
+	const string_t	strBreaks = CEditDoc::GetBreakString(pDocument->GetBreakType());
+	list<length_t>	listLines;
 
 	// sʒũXg
-	size_t	iChar = 0, iLast = 0;
+	length_t	iChar = 0, iLast = 0;
 	while(true) {
 		iChar = strText.find_first_of(CEditDoc::m_wszBreakChars, iLast);
-		if(iChar == wstring::npos)
+		if(iChar == string_t::npos)
 			break;
 		iLast = iChar + ((iChar < strText.length() - 1
 			&& strText[iChar] == L'\r' && strText[iChar == L'\n']) ? 2 : 1);
@@ -539,29 +654,29 @@ void CEditPoint::InsertBox(const wstring& strText) throw(EViewIsAlreadyInavailab
 	if(*listLines.begin() != 0)
 		listLines.push_front(0);
 
-	unsigned long	iLine = m_iLine;	// }s
-	wstring			strLine;			// }sƂȂ镔
+	length_t		iLine = m_iLine;	// }s
+	string_t		strLine;			// }sƂȂ镔
 	unsigned long	xFirstLineInsert;	// }擪s̑}ʒu (r[ x W)
-	for(list<size_t>::const_iterator it = listLines.begin(); it != listLines.end(); ++it) {
+	for(list<length_t>::const_iterator it = listLines.begin(); it != listLines.end(); ++it) {
 		if(it == listLines.begin())	// ŏ̍s
 			xFirstLineInsert = m_pView->PosFromChar(*this).x - m_pView->m_layoutInfo.nLeftTabWidth;
 		if(++it == listLines.end())	// ŏIs
 			strLine = strText.substr(*(--it), wstring::npos);
 		else {
-			size_t	cchLine = *it - ((strText[*it - 1] == L'\n' && strText[*it - 2] == L'\r') ? 2 : 1);
+			length_t	cchLine = *it - ((strText[*it - 1] == L'\n' && strText[*it - 2] == L'\r') ? 2 : 1);
 			cchLine -= *(--it);
 			strLine = strText.substr(*it, cchLine);
 		}
 		++it;
 		if(iLine >= cLines - 1 && it != listLines.end())	// }ʒuɍs݂Ȃꍇ͍쐬
-			strLine += pwszBreaks;
+			strLine += strBreaks;
 		--it;
 
 		if(strLine.length() != 0) {
 			strLine = m_pView->CalculateSpacesReachingVirtualPoint(iLine, xFirstLineInsert) + strLine;	// }ʒu̒
 			*this = pDocument->InsertText(this, CCharPos(iLine,
 				m_pView->CharFromPixel(iLine, xFirstLineInsert,
-					!m_pView->m_ModeState.bCaretMarkByMouse)), strLine);
+					!m_pView->m_modeState.bCaretExtenderByMouse)), strLine);
 		}
 		++iLine;
 	}
@@ -608,7 +723,7 @@ bool CEditPoint::IsStartOfLine() const throw(EViewIsAlreadyInavailable) {
  *	̍sɈړ
  *	@param cLines	ړs
  */
-void CEditPoint::LineDown(unsigned long cLines /* = 1 */) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::LineDown(length_t cLines /* = 1 */) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
@@ -621,7 +736,7 @@ void CEditPoint::LineDown(unsigned long cLines /* = 1 */) throw(EViewIsAlreadyIn
  *	O̍sɈړ
  *	@param cLines	ړs
  */
-void CEditPoint::LineUp(unsigned long cLines /* = 1 */) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::LineUp(length_t cLines /* = 1 */) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
@@ -634,17 +749,17 @@ void CEditPoint::LineUp(unsigned long cLines /* = 1 */) throw(EViewIsAlreadyInav
  *	hLg擪̕Ŏw肵ʒuɈړ
  *	@param nOffset	hLg擪̕
  */
-void CEditPoint::MoveToAbsoluteCharOffset(unsigned long nOffset) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::MoveToAbsoluteCharOffset(length_t nOffset) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
 	const CEditDoc*	pDoc = m_pView->GetDocument();
-	unsigned long	cRead = 0;
+	length_t		cRead = 0;
 	LineIterator	itLines = pDoc->GetLineIterator(0);
-	unsigned long	cLines = pDoc->GetLineCount();
+	length_t		cLines = pDoc->GetLineCount();
 
-	for(unsigned long iLine = 0; iLine < cLines; ++iLine, ++itLines) {
-		unsigned long	cchLine = itLines->GetLine().length() + 1;
+	for(length_t iLine = 0; iLine < cLines; ++iLine, ++itLines) {
+		const length_t	cchLine = itLines->GetLine().length() + 1;
 		if(cRead + cchLine >= nOffset) {
 			m_iLine = iLine;
 			m_iChar = cRead + cchLine - nOffset;
@@ -683,8 +798,8 @@ void CEditPoint::MoveToNextBookmark() throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
-	unsigned long	iLine;
-	unsigned long	cLines = m_pView->GetDocument()->GetLineCount();
+	length_t	iLine;
+	length_t	cLines = m_pView->GetDocument()->GetLineCount();
 
 	// T
 	for(iLine = m_iLine + 1; iLine < cLines; ++iLine) {
@@ -726,7 +841,7 @@ void CEditPoint::MoveToPrevBookmark() throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
-	unsigned long	iLine = m_iLine - 1;
+	length_t	iLine = m_iLine - 1;
 
 	// T
 	while(true) {
@@ -776,9 +891,9 @@ void CEditPoint::MoveToStartOfLine(bool bToFirstText) throw(EViewIsAlreadyInavai
 	if(!bToFirstText)
 		m_iChar = 0;
 	else {
-		const wstring&	strLine = pDocument->GetLine(m_iLine);
-		unsigned long	iFirstText = CEditView::IsWhiteSpace(strLine.c_str(), strLine.length());
-		m_iChar = (m_iChar != iFirstText) ? iFirstText : 0;
+		const length_t	cchHeadSpace = m_pView->GetLexer()->IsWhiteSpace(
+			pDocument->GetLine(m_iLine).c_str(), pDocument->GetLineLength(m_iLine), true);
+		m_iChar = (m_iChar != cchHeadSpace) ? cchHeadSpace : 0;
 	}
 	UpdateLastCx();
 }
@@ -789,17 +904,17 @@ void CEditPoint::NewLine() throw(EViewIsAlreadyInavailable) {
 	VERIFY_VIEW();
 
 	CEditDoc*	pDoc = m_pView->GetDocument();
-	wstring		strBreak = CEditDoc::GetBreakString(pDoc->GetBreakType());
+	string_t	strBreak = CEditDoc::GetBreakString(pDoc->GetBreakType());
 	bool		bDoneSmartIndent = false;	// X}[gCfg
 
 	if(pDoc->IsReadOnly())
 		return;
-//	if(m_ModeState.aitAutoIndentType == AIT_SMART)
+//	if(m_modeState.aitAutoIndentType == AIT_SMART)
 //		bDoneSmartIndent = SmartIndent(L'\n');
 	if(!bDoneSmartIndent
-			&& m_pView->m_ModeState.aitAutoIndentType == AIT_BLOCK) {	// I[gCfg
-		const wstring&	strCurrentLine = pDoc->GetLine(m_iLine);
-		unsigned long	cch = CEditView::IsWhiteSpace(strCurrentLine.c_str(), strCurrentLine.length());
+			&& m_pView->m_modeState.aitAutoIndentType == AIT_BLOCK) {	// I[gCfg
+		const string_t&	strCurrentLine = pDoc->GetLine(m_iLine);
+		const length_t	cch = m_pView->GetLexer()->IsWhiteSpace(strCurrentLine.c_str(), strCurrentLine.length(), true);
 		strBreak += strCurrentLine.substr(0, cch);
 	}
 	if(!bDoneSmartIndent)
@@ -863,7 +978,7 @@ void CEditPoint::OnUpdateDocument(const TUpdateInfo& udi) {
  *	̃y[WɈړ
  *	@param cPages	ړy[W
  */
-void CEditPoint::PageDown(unsigned long cPages /* = 1 */) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::PageDown(length_t cPages /* = 1 */) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
@@ -877,7 +992,7 @@ void CEditPoint::PageDown(unsigned long cPages /* = 1 */) throw(EViewIsAlreadyIn
  *	Õy[WɈړ
  *	@param cPages	ړy[W
  */
-void CEditPoint::PageUp(unsigned long cPages /* = 1 */) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::PageUp(length_t cPages /* = 1 */) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
@@ -891,7 +1006,7 @@ void CEditPoint::PageUp(unsigned long cPages /* = 1 */) throw(EViewIsAlreadyInav
  *	͈͓̃eLXgNbv{[h̓eŒu
  *	@param cChars	1̈ʒu܂ł̕ (ł悢)
  */
-void CEditPoint::Paste(long cChars /* = 0 */) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::Paste(signed_length_t cChars /* = 0 */) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
@@ -932,8 +1047,8 @@ void CEditPoint::Paste(const CCharPos& pos) throw(EViewIsAlreadyInavailable) {
 	if(m_pView->GetDocument()->IsReadOnly())
 		return;
 
-	size_t		cch;
-	wchar_t*	pwszText = 0;
+	length_t	cch;
+	char_t*		pwszText = 0;
 	const UINT	nAvailableClipFormat = m_pView->CanPaste();
 
 	if(nAvailableClipFormat == 0)
@@ -944,7 +1059,7 @@ void CEditPoint::Paste(const CCharPos& pos) throw(EViewIsAlreadyInavailable) {
 	pwszText = new wchar_t[cch + 1];
 	CClipboard::ReadClipboardText(m_pView->m_hWnd, pwszText, cch + 1);
 
-	if(nAvailableClipFormat == ::RegisterClipboardFormatW(L"MSDEVColumnSelect"))
+	if(nAvailableClipFormat == ::RegisterClipboardFormatW(RECTANGLE_TEXT_CLIP_FORMAT))
 		InsertBox(pwszText);
 	else
 		Insert(pwszText);
@@ -976,6 +1091,135 @@ CCharPos CEditPoint::TabIndent(const CCharPos& pos, long nLevel /* = 1 */) throw
 	return Indent(pos, L'\t', nLevel);
 }
 
+/**
+ *	@brief	O̕ւ
+ *
+ *	̃\bh͕NX^PʂŃeLXgւB
+ *	݈ʒuNX^̐擪łȂΏ͎sB
+ *	ւΏۂݍsɖꍇ͎s
+ *
+ *	@return	ւȂꍇ false
+ */
+bool CEditPoint::TransposeChars() {
+	AssertValid();
+	VERIFY_VIEW();
+
+	if(m_pView->GetDocument()->IsReadOnly())
+		return false;
+	if(m_pView->GetDocument()->GetLineLength(m_iLine) < 2	// ւ镶A݈ʒuNX^̐擪łȂ
+			|| !CBoundarySearcher::IsFirstCharacterOfCluster(GetCodePoint()))
+		return false;
+
+	const CCharPos	posNextCluster = m_pView->m_pBoundarySearcher->SearchFirstCharacterOfCluster(*this, true);
+	if(posNextCluster.m_iLine != m_iLine	// ̃NX^sɖ
+			|| posNextCluster == *this)		// ʒu
+		return false;
+	const CCharPos	posPrevCluster = m_pView->m_pBoundarySearcher->SearchFirstCharacterOfCluster(*this, false);
+	if(posPrevCluster.m_iLine != m_iLine	// ÕNX^sɖ
+			|| posPrevCluster == *this)		// ʒu
+		return false;
+
+	const string_t	str = GetText(posNextCluster);
+	Delete(posNextCluster);
+	m_iChar = posPrevCluster.m_iChar;
+	Insert(str);
+
+	return true;
+}
+
+/**
+ *	ݍsƎ̍s (ݍsŏIs̏ꍇ͑O̍s) ւ
+ *	@param	ւȂꍇ false
+ */
+bool CEditPoint::TransposeLines() {
+	AssertValid();
+	VERIFY_VIEW();
+
+	if(m_pView->GetDocument()->IsReadOnly())
+		return false;
+
+	const CEditDoc*	pDocument = m_pView->GetDocument();
+	const length_t	cLines = pDocument->GetLineCount();
+
+	if(cLines < 2)	// 1s
+		return false;
+
+	const length_t	iOrgChar = m_iChar;
+	if(cLines - 1 != m_iLine) {
+		const CCharPos	posPair(m_iLine + 1, 0);
+		m_iChar = 0;
+		const string_t	str = GetText(posPair);
+		Delete(posPair);
+		++m_iLine;
+		m_iChar = 0;
+		Insert(str);
+		MoveToPoint(CCharPos(m_iLine - 1, iOrgChar));
+	} else {
+		const CCharPos	posPair(m_iLine - 1, pDocument->GetLineLength(m_iLine - 1));
+		m_iChar = pDocument->GetLineLength(m_iLine);
+		const string_t	str = GetText(posPair);
+		Delete(posPair);
+		--m_iLine;
+		m_iChar = pDocument->GetLineLength(m_iLine);
+		Insert(str);
+		MoveToPoint(CCharPos(m_iLine, iOrgChar));
+	}
+
+	return true;
+}
+
+/**
+ *	@brief	O̒Pւ
+ *
+ *	ւΏۂݍsɖꍇA͎s (̍s̒PƂ̌͋NȂ)
+ *
+ *	@return	ւȂꍇ false
+ */
+bool CEditPoint::TransposeWords() {
+	AssertValid();
+	VERIFY_VIEW();
+
+	if(m_pView->GetDocument()->IsReadOnly())
+		return false;
+	const string_t	strLine = m_pView->GetDocument()->GetLine(m_iLine);
+	if(strLine.length() < 2 || m_iChar == strLine.length())
+		return false;
+
+	// ւ2̒P̊JnʒuATPʒu߂B
+	// O2_̓Aǂ炩1݈ʒuƓɂȂ\
+	const BoundaryPosition	BP_WORDHEAD = static_cast<BoundaryPosition>(BP_START | BP_ALPHANUM | BP_NOANOTHERLINE);
+	const BoundaryPosition	BP_WORDEND = static_cast<BoundaryPosition>(BP_END | BP_ALPHANUM | BP_NOANOTHERLINE);
+	const CBoundarySearcher&	searcher = *m_pView->m_pBoundarySearcher;
+	const CCharPos	pos1stWordHead = searcher.SearchWordBoundary(CCharPos(m_iLine, m_iChar), false, BP_WORDHEAD);
+	const CCharPos	pos1stWordEnd = searcher.SearchWordBoundary(*this, true, BP_WORDEND);
+
+	if(m_iLine != pos1stWordHead.m_iLine || m_iLine != pos1stWordEnd.m_iLine)	// PꋫEsɖ
+		return false;
+	if(*this < pos1stWordHead || *this > pos1stWordEnd)	// ݈ʒuPłȂ
+		return false;
+
+	const CCharPos	pos2ndWordEnd = searcher.SearchWordBoundary(pos1stWordEnd, true, BP_WORDEND);
+
+	if(m_iLine != pos2ndWordEnd.m_iLine)	// Ώۂsɖ
+		return false;
+
+	const CCharPos	pos2ndWordHead = searcher.SearchWordBoundary(pos1stWordEnd, true, BP_WORDHEAD);
+
+	const length_t	iOrgChar = m_iChar;
+	m_iChar = pos1stWordHead.m_iChar;
+	const string_t	str1 = GetText(pos1stWordEnd);
+	Delete(pos1stWordEnd);
+	m_iChar = pos2ndWordHead.m_iChar - (pos1stWordEnd.m_iChar - pos1stWordHead.m_iChar);
+	Insert(str1);
+	const string_t	str2 = GetText(CCharPos(m_iLine, m_iChar + (pos2ndWordEnd.m_iChar - pos2ndWordHead.m_iChar)));
+	Delete(CCharPos(m_iLine, m_iChar + (pos2ndWordEnd.m_iChar - pos2ndWordHead.m_iChar)));
+	m_iChar = pos1stWordHead.m_iChar;
+	Insert(str2);
+	MoveToPoint(CCharPos(m_iLine, iOrgChar + (pos2ndWordEnd.m_iChar - pos1stWordEnd.m_iChar)));
+
+	return true;
+}
+
 ///	m_cxLast ݈ʒuɍXV
 void CEditPoint::UpdateLastCx() {
 	AssertValid();
@@ -988,14 +1232,12 @@ void CEditPoint::UpdateLastCx() {
  *	̒P̖Ɉړ
  *	@param cWords	ړPꐔ
  */
-void CEditPoint::WordEndNext(unsigned long cWords /* = 1 */) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::WordEndNext(length_t cWords /* = 1 */) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
-	unsigned long	cLines = m_pView->GetDocument()->GetLineCount();
-
 	Normalize();
-	*this = m_pView->m_pBoundarySearcher->SearchWordBoundary(*this, true, true);
+	*this = m_pView->m_pBoundarySearcher->SearchWordBoundary(*this, true, BP_END);
 	UpdateLastCx();
 }
 
@@ -1003,12 +1245,12 @@ void CEditPoint::WordEndNext(unsigned long cWords /* = 1 */) throw(EViewIsAlread
  *	O̒P̖Ɉړ
  *	@param cWords	ړPꐔ
  */
-void CEditPoint::WordEndPrev(unsigned long cWords /* = 1 */) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::WordEndPrev(length_t cWords /* = 1 */) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
 	Normalize();
-	*this = m_pView->m_pBoundarySearcher->SearchWordBoundary(*this, false, true);
+	*this = m_pView->m_pBoundarySearcher->SearchWordBoundary(*this, false, BP_END);
 	UpdateLastCx();
 }
 
@@ -1016,14 +1258,12 @@ void CEditPoint::WordEndPrev(unsigned long cWords /* = 1 */) throw(EViewIsAlread
  *	̒P̐擪Ɉړ
  *	@param cWords	ړPꐔ
  */
-void CEditPoint::WordNext(unsigned long cWords /* = 1 */) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::WordNext(length_t cWords /* = 1 */) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
-	unsigned long	cLines = m_pView->GetDocument()->GetLineCount();
-
 	Normalize();
-	*this = m_pView->m_pBoundarySearcher->SearchWordBoundary(*this, true, false);
+	*this = m_pView->m_pBoundarySearcher->SearchWordBoundary(*this, true, BP_START);
 	UpdateLastCx();
 }
 
@@ -1031,12 +1271,12 @@ void CEditPoint::WordNext(unsigned long cWords /* = 1 */) throw(EViewIsAlreadyIn
  *	O̒P̐擪Ɉړ
  *	@param cWords	ړPꐔ
  */
-void CEditPoint::WordPrev(unsigned long cWords /* = 1 */) throw(EViewIsAlreadyInavailable) {
+void CEditPoint::WordPrev(length_t cWords /* = 1 */) throw(EViewIsAlreadyInavailable) {
 	AssertValid();
 	VERIFY_VIEW();
 
 	Normalize();
-	*this = m_pView->m_pBoundarySearcher->SearchWordBoundary(*this, false, false);
+	*this = m_pView->m_pBoundarySearcher->SearchWordBoundary(*this, false, BP_START);
 	UpdateLastCx();
 }
 
@@ -1089,40 +1329,4 @@ void CVisibleEditPoint::OnUpdateDocument(const TUpdateInfo& udi) {
 	CEditPoint::OnUpdateDocument(udi);
 }
 
-
-// CEditRange class implementation
-/////////////////////////////////////////////////////////////////////////////
-#if 0
-
-///	͈͓eLXgLs^CY
-void CEditRange::MakeCapitalized() throw(EViewIsAlreadyInavailable) {
-	AssertValid();
-	if(!IsAvailable())
-		throw EViewIsAlreadyInavailable();
-	if(m_pView->GetDocument()->IsReadOnly())
-		return;
-
-	wstring		str = GetText();
-	size_t		cch = str.length();
-	wchar_t*	pwsz = new wchar_t[str.length()];
-
-	for(size_t i = 0; i < cch; ) {
-		size_t	cchWord = m_pView->IsWord(pwsz + i, cch - i);
-		if(cchWord > 0) {
-			size_t	cchHead = ((i < cch - 1)
-						&& CTextConverter::IsUTF16HighSurrogate(pwsz[i])
-						&& CTextConverter::IsUTF16LowSurrogate(pwsz[i + 1])) ? 2 : 1;
-			::CharUpperBuffW(pwsz + i, cchHead);
-			::CharLowerBuffW(pwsz + i + cchHead, cch - i - cchHead);
-			i += cchHead;
-		} else
-			++i;
-	}
-
-	str.assign(pwsz, str.length());
-	Replace(str);
-	delete[] pwsz;
-}
-#endif
-
 /* [EOF] */
\ No newline at end of file
