@@ -1,10 +1,11 @@
 // KeywordManager.h
-// (c) 2003 exeal
+// (c) 2003-2004 exeal
 
 #ifndef _KEYWORD_MANAGER_H_
 #define _KEYWORD_MANAGER_H_
 
-#include <string>
+#include "AscensionCommon.h"
+#include "..\..\Manah\Object.h"
 #include <vector>
 
 
@@ -14,7 +15,7 @@ namespace Ascension {
 /////////////////////////////////////////////////////////////////////////////
 
 ///	L[[h̊Ǘ
-class CKeywordManager {
+class CKeywordManager : public Manah::CObject {
 	// RXgN^
 public:
 	CKeywordManager();
@@ -23,13 +24,13 @@ public:
 
 	// \bh
 public:
-	void	Add(const std::wstring& strKeyword);								// L[[h̒ǉ
+	void	Add(const string_t& strKeyword);									// L[[h̒ǉ
 	void	Clear();															// L[[h̑S폜
-	bool	Find(const wchar_t* strText, size_t cch, bool bIgnoreCase) const;	// L[[ȟ
+	bool	Find(const char_t* strText, length_t cch, bool bIgnoreCase) const;	// L[[ȟ
 
 	// f[^o
 private:
-	std::vector<wchar_t*>	m_vecKeywords;	// \L[[h
+	std::vector<char_t*>	m_vecKeywords;	// \L[[h
 };
 
 } // namespace Ascension
