@@ -15,6 +15,9 @@
 #   space -- Zs
 #   format -- Cf
 #   id_continue -- Mn, Mc, Nd, Pc
+#   punctuation -- Pc, Pd, Ps, Pe, Pi, Pf, Po
+#   symbol -- Sm, Sc, Sk, So
+#   control -- Cc, Cf
 #
 # ɸϡ
 
@@ -40,6 +43,12 @@ if($ARGV[0] eq 'mark') {
 	$strPattern = 'Cf';
 } elsif($ARGV[0] eq 'id_continue') {
 	$strPattern = '(Mn)|(Mc)|(Nd)|(Pc)';
+} elsif($ARGV[0] eq 'punctuation') {
+	$strPattern = '(Pc)|(Pd)|(Ps)|(Pe)|(Pi)|(Pf)|(Po)';
+} elsif($ARGV[0] eq 'symbol') {
+	$strPattern = '(Sm)|(Sc)|(Sk)|(So)';
+} elsif($ARGV[0] eq 'control') {
+	$strPattern = '(Cc)|(Cf)';
 } else {
 	$strPattern = $ARGV[0];
 }
