@@ -1,5 +1,5 @@
 // BookmarkDlg.h
-// (c) 2003 exeal
+// (c) 2003-2004 exeal
 
 #ifndef _BOOKMARK_DLG_H_
 #define _BOOKMARK_DLG_H_
@@ -17,20 +17,15 @@ class CAlphaApp;
 
 ///	[ubN}[N] _CAO
 class CBookmarkDlg : public CDialog {
-	// f[^o
-private:
-	const CAlphaApp*			m_pApp;
-	map<wstring, unsigned int>	m_mapTabIndex;	// t@C -> ^uԍ
-
 	// RXgN^
 public:
-	CBookmarkDlg(const CAlphaApp* pApp);
+	CBookmarkDlg(const CAlphaApp& app);
 
 	// \bh
 public:
 	void	UpdateList();
 protected:
-	void	GetItemInfo(unsigned long iItem, wstring& strFileName, unsigned long& iLine) const;
+	void	GetItemInfo(unsigned long iItem, wstring& strFileName, length_t& iLine) const;
 	void	OnBtnDelete();
 
 	// bZ[Wnh
@@ -39,6 +34,11 @@ protected:
 	virtual bool	OnInitDialog(HWND hwndFocus, LPARAM lInitParam);		// WM_INITDIALOG
 	virtual bool	OnNotify(int idCtrl, LPNMHDR lpNMHDR);					// WM_NOTIFY
 	virtual void	OnOK();													// IDOK
+
+	// f[^o
+private:
+	const CAlphaApp&			m_app;
+	map<wstring, unsigned int>	m_mapTabIndex;	// t@C -> ^uԍ
 };
 
 }
