@@ -1,10 +1,12 @@
 // CodePagesDlg.h
-// (c) 2003 exeal
+// (c) 2003-2004 exeal
 
 #ifndef _CODE_PAGES_DLG_H_
 #define _CODE_PAGES_DLG_H_
 
 #include "..\Manah\Dialog.h"
+#include "..\Manah\Encoder.h"
+using Manah::Windows::Controls::CDialog;
 
 
 // CCodePagesDlg class definition
@@ -16,13 +18,13 @@ namespace Alpha {
 class CCodePagesDlg : public CDialog {
 	// f[^o
 public:
-	UINT	m_nCodePage;	// IꂽR[hy[W
+	Manah::Text::CodePage	m_nCodePage;	// IꂽR[hy[W
 
 	// bZ[Wnh
 protected:
-	virtual bool	OnInitDialog(HWND hwndFocus, LPARAM lInitParam);	// WM_INITDIALOG
-	virtual bool	OnNotify(int idCtrl, LPNMHDR lpNMHDR);				// WM_NOTIFY
-	virtual void	OnOK();												// IDOK
+	virtual	bool	OnCommand(WORD wID, WORD wNotifyCode, HWND hwndCtrl);	// WM_COMMAND
+	virtual bool	OnInitDialog(HWND hwndFocus, LPARAM lInitParam);		// WM_INITDIALOG
+	virtual void	OnOK();													// IDOK
 };
 
 }
