@@ -6,6 +6,8 @@
 #include "..\Manah\Dialog.h"
 #include "..\Manah\LinkLabel.h"
 #include <vector>
+using Manah::Windows::Controls::CDialog;
+using Manah::Windows::Controls::CLinkLabel;
 
 
 // CConfirmUnsavedDocumentDlg class definition
