@@ -16,7 +16,7 @@ namespace Alpha {
 
 ///	IDebugExpressionCallBack ̕WIȎ
 class CDebugExpressionCallBack :
-		public IUnknownImpl<CMultiThreadMemoryManageModel>,
+		public IUnknownImpl<MultiThreadedRefCount>,
 		public IDebugExpressionCallBack {
 	// RXgN^
 public:
