@@ -48,8 +48,8 @@ bool CExecuteCommandDlg::OnCommand(WORD wID, WORD wNotifyCode, HWND hwndCtrl) {
 			::EnableWindow(GetDlgItem(IDOK), ::GetWindowTextLength(GetDlgItem(wID)) != 0);
 		break;
 	case IDC_BTN_CLEAR:	// [NA]
-		m_oDosCommand.Clear();
-		OnChangeOutput();
+		SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_SETSEL, 0, -1);
+		SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_REPLACESEL, false, reinterpret_cast<LPARAM>(L""));
 		break;
 	}
 
@@ -74,7 +74,7 @@ void CExecuteCommandDlg::OnOK() {
 		wchar_t*		pwszCmdLine = 0;
 		size_t			cchCmdLine;
 		wstring			strCmdLine;			// `̃R}hC
-		const wstring&	strFilePath = m_pApplication->GetActiveDocument()->GetPathName();
+		const wstring&	strFilePath = m_pApplication->GetActiveTab()->GetFilePath();
 
 		hwndCmdLineCmb = GetDlgItem(IDC_COMBO_COMMAND);
 		cchCmdLine = ::GetWindowTextLength(hwndCmdLineCmb);
@@ -101,16 +101,16 @@ void CExecuteCommandDlg::OnOK() {
 
 			::EnableWindow(GetDlgItem(IDC_COMBO_COMMAND), false);
 			::EnableWindow(hInputEdit, true);
-			::SetWindowTextW(GetDlgItem(IDOK), L"(&I)");
-			::SetWindowTextW(GetDlgItem(IDCANCEL), L"~(&A)");
+			SetDlgItemText(IDOK, L"(&I)");
+			SetDlgItemText(IDCANCEL, L"~(&A)");
 			GetWindowRect(&rect);
 			rect.bottom = rect.top + 460;
 			MoveWindow(&rect, true);
 			::EnableWindow(GetDlgItem(IDC_CHK_GETCONSOLE), false);
 			::EnableWindow(GetDlgItem(IDC_CHK_USEUNICODEFORINPUT), false);
-			::MoveWindow(hSplitStatic, 10, 112, 452, 2, false);
-			::MoveWindow(hInputEdit, 10, 118, 452, 16, false);
-			::MoveWindow(hOutputEdit, 10, 138, 452, 278, false);
+			::MoveWindow(hSplitStatic, 10, 122, 466, 2, false);
+			::MoveWindow(hInputEdit, 10, 130, 466, 18, false);
+			::MoveWindow(hOutputEdit, 10, 150, 466, 270, false);
 			::ShowWindow(hSplitStatic, SW_SHOW);
 			::ShowWindow(hInputEdit, SW_SHOW);
 			::ShowWindow(hOutputEdit, SW_SHOW);
@@ -151,43 +151,11 @@ void CExecuteCommandDlg::OnOK() {
 //	CDialog::OnOK();
 }
 
-///	@see CDosCommand::OnChangeOutput
-void CExecuteCommandDlg::OnChangeOutput() {
-	HWND			hwndOutputEdit = GetDlgItem(IDC_EDIT_OUTPUT);
-	string			strOutputOrg = m_oDosCommand.GetOutputLines();
-	stringstream	ssOutput;
-	int				cchOutput;
-	wchar_t*		pwszOutput = 0;
-
-	// LF ȂGfBbgRg[̂߂ CR+LF ɕϊ
-	// (̓̓Rg[gꍇ͕ϊȂĂ낤)
-	string::size_type	i = 0, iLast = 0;
-	while(true) {
-		i = strOutputOrg.find('\n', iLast);
-		if(i == string::npos) {
-			ssOutput << strOutputOrg.substr(iLast);
-			break;
-		} else if(i == 0)
-			ssOutput << "\r\n";
-		else if(strOutputOrg[i - 1] == '\r')
-			ssOutput << strOutputOrg.substr(iLast, i - iLast + 1);
-		else
-			ssOutput << strOutputOrg.substr(iLast, i - iLast) << "\r\n";
-		iLast = i + 1;
-	}
-
-	// ݂ ANSI R[hy[WƂO Unicode ɕϊ
-	cchOutput = ::MultiByteToWideChar(CP_ACP, 0, ssOutput.str().c_str(), -1, 0, 0);
-	pwszOutput = new wchar_t[cchOutput * sizeof(wchar_t)];
-	::MultiByteToWideChar(CP_ACP, 0, ssOutput.str().c_str(), -1, pwszOutput, cchOutput);
-
-	::SendMessage(hwndOutputEdit, EM_SETSEL, 0, -1);
-	::SendMessage(hwndOutputEdit, EM_REPLACESEL, 0, reinterpret_cast<LPARAM>(pwszOutput));
-	delete[] pwszOutput;
-}
-
-///	@see CDosCommand::OnExit
+///	@see IDosCommandEvent::OnExit
 void CExecuteCommandDlg::OnExit(CDosCommand::ExecuteStatus es) {
+	if(!IsWindow())	// sɃoc{^Ƃɗ
+		return;
+
 	SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_SETSEL, -1, -1);
 	switch(es) {
 	case CDosCommand::timeout:
@@ -213,18 +181,49 @@ void CExecuteCommandDlg::OnExit(CDosCommand::ExecuteStatus es) {
 	}
 	m_bExecuting = false;
 
-	HWND	hwndOKButton = GetDlgItem(IDOK);
-
 	::EnableWindow(GetDlgItem(IDC_CHK_GETCONSOLE), true);
 	::EnableWindow(GetDlgItem(IDC_CHK_USEUNICODEFORINPUT), true);
 	::EnableWindow(GetDlgItem(IDC_COMBO_COMMAND), true);
 	::EnableWindow(GetDlgItem(IDC_EDIT_INPUT), false);
 	::SetFocus(GetDlgItem(IDC_COMBO_COMMAND));
 	SendDlgItemMessage(IDC_COMBO_COMMAND, CB_SETEDITSEL, 0, -1);
-	::SetWindowText(hwndOKButton, L"s(&E)");
-	::SetWindowText(GetDlgItem(IDCANCEL), L"");
+	SetDlgItemText(IDOK, L"s(&E)");
+	SetDlgItemText(IDCANCEL, L"");
 	SetWindowText(L"R}h̎s");
 }
 
+///	@see IDosCommandEvent::OnReceiveOutput
+void CExecuteCommandDlg::OnReceiveOutput(const string& strOutput) {
+	stringstream	ssOutput;
+	int				cchOutput;
+	wchar_t*		pwszOutput = 0;
+
+	// LF ȂGfBbgRg[̂߂ CR+LF ɕϊ
+	// (̓̓Rg[gꍇ͕ϊȂĂ낤)
+	string::size_type	i = 0, iLast = 0;
+	while(true) {
+		i = strOutput.find('\n', iLast);
+		if(i == string::npos) {
+			ssOutput << strOutput.substr(iLast);
+			break;
+		} else if(i == 0)
+			ssOutput << "\r\n";
+		else if(strOutput[i - 1] == '\r')
+			ssOutput << strOutput.substr(iLast, i - iLast + 1);
+		else
+			ssOutput << strOutput.substr(iLast, i - iLast) << "\r\n";
+		iLast = i + 1;
+	}
+
+	// ݂ ANSI R[hy[WƂO Unicode ɕϊ
+	cchOutput = ::MultiByteToWideChar(CP_ACP, 0, ssOutput.str().c_str(), -1, 0, 0);
+	pwszOutput = new wchar_t[cchOutput * sizeof(wchar_t)];
+	::MultiByteToWideChar(CP_ACP, 0, ssOutput.str().c_str(), -1, pwszOutput, cchOutput);
+
+	// ɒǉ
+	SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_SETSEL, -1, -1);
+	SendDlgItemMessage(IDC_EDIT_OUTPUT, EM_REPLACESEL, false, reinterpret_cast<LPARAM>(pwszOutput));
+	delete[] pwszOutput;
+}
 
 /* [EOF] */
\ No newline at end of file
