@@ -6,7 +6,8 @@
 
 #include "..\Manah\Dialog.h"
 #include "..\Manah\DosCommand.h"
-using namespace Manah::Windows;
+using Manah::Windows::Controls::CDialog;
+using Manah::Windows::CDosCommand;
 
 
 namespace Alpha {
@@ -17,7 +18,9 @@ class CAlphaApp;
 /////////////////////////////////////////////////////////////////////////////
 
 ///	[R}h̎s] _CAO
-class CExecuteCommandDlg : public CDialog, public CDosCommand::ICommandEvent {
+class CExecuteCommandDlg :
+		public CDialog,
+		virtual public CDosCommand::ICommandEvent {
 	// f[^o
 private:
 	CAlphaApp*	m_pApplication;
@@ -36,8 +39,8 @@ public:
 	// \bh
 public:
 	// CDosCommand::ICommandEvent C^[tFCX
-	virtual void	OnChangeOutput();
 	virtual void	OnExit(CDosCommand::ExecuteStatus es);
+	virtual void	OnReceiveOutput(const string& strOutput);
 
 	// bZ[Wnh
 protected:
