@@ -1,5 +1,5 @@
 // GotoLineDlg.h
-// (c) 2003 exeal
+// (c) 2003-2004 exeal
 
 #ifndef _GOTO_LINE_DLG_H_
 #define _GOTO_LINE_DLG_H_
@@ -15,24 +15,26 @@ namespace Alpha {
 class CAlphaApp;
 
 ///	[wsֈړ] _CAO
-class CGotoLineDlg : public CDialog {
+class CGotoLineDlg : public Manah::Windows::Controls::CDialog {
 	// f[^o
 public:
-	const CAlphaApp*	m_pApp;
 	bool				m_bLineUnit;		// _ŝƂ true
 	bool				m_bSaveSelection;	// IێƂ true
 	bool				m_bAutoClose;		// Ƀ_CAOƂ true
 	unsigned long		m_iLineNumber;		// sԍ
+private:
+	const CAlphaApp&	m_app;
 
 	// RXgN^
 public:
-	CGotoLineDlg(const CAlphaApp* pApp);
+	CGotoLineDlg(const CAlphaApp& app);
 
 	// bZ[Wnh
 protected:
-	virtual void	OnClose();												// WM_CLOSE
-	virtual bool	OnInitDialog(HWND hwndFocus, LPARAM lInitParam);		// WM_INITDIALOG
-	virtual void	OnOK();													// IDOK
+	void	OnActivate(UINT nState, HWND hwndPrevious, bool bMinimized);	// WM_ACTIVATE
+	void	OnClose();														// WM_CLOSE
+	bool	OnInitDialog(HWND hwndFocus, LPARAM lInitParam);				// WM_INITDIALOG
+	void	OnOK();															// IDOK
 };
 
 }
