@@ -12,7 +12,8 @@ using Alpha::CMRUManager;
 ///	RXgN^
 CMRUManager::CMRUManager(unsigned short cLimit, int nStartID, bool bOwnerDrawMenu /* =false */)
 		: m_cLimit(cLimit), m_nStartID(nStartID), m_bOwnerDraw(bOwnerDrawMenu) {
-	m_pPopupMenu = new CPopupMenu();
+	m_pPopupMenu = new CMenu();
+	_ReconstructMenu();
 }
 
 ///	fXgN^
@@ -26,10 +27,17 @@ void CMRUManager::_ReconstructMenu() {
 	list<wstring>::const_iterator	it = m_listFileNames.begin();
 	wchar_t							szItem[MAX_PATH + 2];
 
-	m_pPopupMenu->ResetContent();
+	delete m_pPopupMenu;
+	m_pPopupMenu = new CMenu();
+	if(m_listFileNames.empty()) {	// ̏ꍇ
+		m_pPopupMenu->AppendMenuItem(0, MFT_STRING, MFS_GRAYED, L"(Ȃ)");
+		return;
+	}
 	while(it != m_listFileNames.end()) {
 		swprintf(szItem, L"&%x  %s", iItem, it->c_str());
-		m_pPopupMenu->AppendMenuItem(m_nStartID + iItem, m_bOwnerDraw ? MF_OWNERDRAW : MF_STRING, szItem);
+		m_pPopupMenu->AppendMenuItem(m_nStartID + iItem,
+			m_bOwnerDraw ? MFT_OWNERDRAW : MFT_STRING,
+			m_bOwnerDraw ? 0 : szItem);
 		++it;
 		++iItem;
 	}
@@ -72,7 +80,8 @@ void CMRUManager::Add(const wstring& strFilePath) {
 ///	ڂSč폜
 void CMRUManager::Clear() {
 	m_listFileNames.clear();
-	m_pPopupMenu->ResetContent();
+	delete m_pPopupMenu;
+	m_pPopupMenu = new CMenu();
 	_ReconstructMenu();
 }
 
@@ -92,7 +101,7 @@ const wstring& CMRUManager::GetFilePathAt(unsigned short i) const throw(out_of_r
 }
 
 ///	|bvAbvj[Ԃ
-CPopupMenu* CMRUManager::GetPopupMenu() const {
+CMenu* CMRUManager::GetPopupMenu() const {
 	return m_pPopupMenu;
 }
 
@@ -113,7 +122,7 @@ void CMRUManager::Remove(unsigned short i) throw(out_of_range) {
 
 ///	ڐ̏l̐ݒ
 void CMRUManager::SetLimit(unsigned short cLimit) {
-	CPopupMenu*	pItem = 0;
+	CMenu*	pItem = 0;
 
 	if(cLimit < 4)
 		cLimit = 4;
@@ -124,7 +133,7 @@ void CMRUManager::SetLimit(unsigned short cLimit) {
 	if(m_listFileNames.size() > cLimit) {
 		m_listFileNames.resize(cLimit);
 		for(unsigned short i = m_listFileNames.size() - 1; i > cLimit; --i)
-			m_pPopupMenu->RemoveMenuItem(i, pItem);
+			m_pPopupMenu->RemoveMenuItem(i, false);
 	}
 }
 
