@@ -5,6 +5,7 @@
 #define _NEW_FILE_FORMAT_DLG_H_
 #include "..\Manah\Dialog.h"
 #include "Ascension\EditDoc.h"	// BreakType
+using Manah::Windows::Controls::CDialog;
 
 
 namespace Alpha {
@@ -16,10 +17,10 @@ namespace Alpha {
 class CNewFileFormatDlg : public CDialog {
 	// f[^o
 public:
-	UINT					m_nCodePage;			// R[hy[W
+	Manah::Text::CodePage	m_nCodePage;			// R[hy[W
 	Ascension::BreakType	m_breakType;			// sR[h
 	list<wstring>			m_listDocumentTypes;	// ^CṽXg
-	unsigned int			m_iDocumentType;		// Iꂽ^Cv
+	size_t					m_iDocumentType;		// Iꂽ^Cv
 
 	// bZ[Wnh
 protected:
