@@ -1,28 +1,38 @@
 // SelectLanguageDlg.h
-// (c) 2003 exeal
+// (c) 2003-2004 exeal
 
 #ifndef _SELECT_LANGUAGE_DLG_H_
 #define _SELECT_LANGUAGE_DLG_H_
 #include "..\Manah\Dialog.h"
 #include <set>
+using Manah::Windows::Controls::CDialog;
 
 
 namespace Alpha {
 
 ///	[GW̑I] _CAO
 class CSelectLanguageDlg : public CDialog {
-	// f[^o
+	// \bh
 public:
-	wstring			m_strSelection;	// Iꂽꖼ
-	set<wstring>	m_setLanguages;	// XgAbv錾
+	wstring	GetSelectedLanguage() const;
 
 	// bZ[Wnh
 protected:
 	virtual bool	OnCommand(WORD wID, WORD wNotifyCode, HWND hwndCtrl);	// WM_COMMAND
 	virtual bool	OnInitDialog(HWND hwndFocus, LPARAM lInitParam);		// WM_INITDIALOG
 	virtual void	OnOK();													// IDOK
+
+	// f[^o
+private:
+	wstring	m_strSelectedLanguage;
 };
 
+
+///	IꂽԂ
+inline wstring CSelectLanguageDlg::GetSelectedLanguage() const {
+	return m_strSelectedLanguage;
+}
+
 }
 
 #endif /* _SELECT_LANGUAGE_DLG_H_ */
