@@ -17,7 +17,7 @@ namespace OLE {
 /// eLXgf[^̃hbOAhhbvɎgp IDataObject B
 /// ANSI AUnicode ̗ɑΉ
 class CDraggingTextDataObject :
-		public IUnknownImpl<CSingleThreadMemoryManageModel>,
+		public IUnknownImpl<SingleThreadedRefCount>,
 		public IDataObject {
 	// RXgN^
 public:
@@ -57,7 +57,7 @@ private:
 
 	/// IDataObject::EnumFormatEtc ̖߂l
 	class CAvailableFormatsEnumerator :
-			public IUnknownImpl<CSingleThreadMemoryManageModel>,
+			public IUnknownImpl<SingleThreadedRefCount>,
 			public IEnumFORMATETC {
 		// RXgN^
 	public:
