@@ -209,16 +209,10 @@ public:
 class CDC : public CMinimalDC {
 };
 
-} /* namespace GDI */
-} /* namespace Windows */
-} /* namespace Manah */
-
 
 // CMinimalDC class implementation
 /////////////////////////////////////////////////////////////////////////////
 
-using Manah::Windows::GDI::CMinimalDC;
-
 inline CMinimalDC::CMinimalDC() : m_hDC(0) {
 }
 
@@ -967,7 +961,10 @@ inline void CMinimalDC::UpdateColors() {
 // CDC class implementation
 /////////////////////////////////////////////////////////////////////////////
 
-using Manah::Windows::GDI::CDC;
+
+} /* namespace GDI */
+} /* namespace Windows */
+} /* namespace Manah */
 
 #endif /* _DC_H_ */
 
