@@ -119,17 +119,10 @@ protected:
 	static LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
 };
 
-} /* namespace Controls */
-} /* namespace Windows */
-} /* namespace Manah */
-
 
 // CListCtrl class implementation
 /////////////////////////////////////////////////////////////////////////////
 
-using Manah::Windows::Controls::CListCtrl;
-using Manah::Windows::Controls::CEdit;
-
 inline SIZE CListCtrl::ApproximateViewRect(SIZE size, int iCount /* = -1 */) const {
 	AssertValidAsWindow();
 	
@@ -695,6 +688,10 @@ inline LRESULT CALLBACK CListCtrl::WndProc(HWND hWnd, UINT message, WPARAM wPara
 	}
 }
 
+} /* namespace Controls */
+} /* namespace Windows */
+} /* namespace Manah */
+
 #endif /* _LIST_CTRL_H_ */
 
 /* [EOF] */
\ No newline at end of file
