@@ -30,7 +30,7 @@ private:
 public:
 	template<class T>
 	CMessageArguments& operator %(const T& rhs) {
-		basic_stringstream<TCHAR> ss;
+		basic_ostringstream<TCHAR> ss;
 		ss << rhs;
 		m_listArgs.push_back(ss.str());
 		return *this;
@@ -41,7 +41,6 @@ private:
 	// f[^o
 private:
   list<tstring> m_listArgs;
-
 };
 
 
@@ -61,7 +60,7 @@ public:
 																					// arguments by space
 	Controls::CWindow*	GetMainWindow() const;
 	int				LoadString(UINT nID, TCHAR* lpszString, int cchMax) const;
-	tstring			LoadString(UINT nID) const;
+	tstring			LoadString(UINT nID, const CMessageArguments& args = CMessageArguments()) const;
 	tstring			LoadMessage(DWORD dwID, const CMessageArguments& args = CMessageArguments()) const;
 	HCURSOR			LoadCursor(UINT nCursorID) const;
 	HCURSOR			LoadCursor(const TCHAR* lpszCursorName) const;
@@ -103,16 +102,10 @@ protected:
 	HACCEL				m_hAcceleratorTable;
 };
 
-} /* namespace Windows */
-} /* namespace Manah */
-
 
 // CWinApp class implementation
 /////////////////////////////////////////////////////////////////////////////
 
-using Manah::Windows::CWinApp;
-using Manah::Windows::CMessageArguments;
-
 inline CWinApp::CWinApp(const Manah::Windows::Controls::CWindow* pWindow /* = 0 */)
 		: m_pMainWindow(const_cast<Manah::Windows::Controls::CWindow*>(pWindow)), m_hAcceleratorTable(0) {
 	TCHAR*	psz = 0;
@@ -131,11 +124,11 @@ inline CWinApp::~CWinApp() {
 
 inline void CWinApp::GetCommandLineArguments(const TCHAR* pszCmdLine, std::vector<tstring>& vecArgs) {
 	assert(pszCmdLine != 0);
-	TCHAR*			pszEach = new TCHAR[_tcslen(pszCmdLine) + 1];
-	TCHAR			ch;
-	size_t			i = 0;
-	size_t			cchEach = 0;
-	bool			bInQuote = false;
+	TCHAR*		pszEach = new TCHAR[_tcslen(pszCmdLine) + 1];
+	TCHAR		ch;
+	std::size_t	i = 0;
+	std::size_t	cchEach = 0;
+	bool		bInQuote = false;
 
 	vecArgs.clear();
 	while(true) {
@@ -279,7 +272,7 @@ inline tstring CWinApp::LoadMessage(DWORD dwID, const CMessageArguments& args /*
 	void*								psz = 0;
 	TCHAR**								pp = new TCHAR*[args.m_listArgs.size()];
 	std::list<tstring>::const_iterator	it;
-	size_t							    i;
+	std::size_t				  		  i;
 
 	for(it = args.m_listArgs.begin(), i = 0; it != args.m_listArgs.end(); ++it, ++i)
 		pp[i] = const_cast<TCHAR*>(it->c_str());
@@ -288,8 +281,8 @@ inline tstring CWinApp::LoadMessage(DWORD dwID, const CMessageArguments& args /*
 		reinterpret_cast<TCHAR*>(&psz), 0, reinterpret_cast<va_list*>(pp));
 	delete[] pp;
 	if(psz != 0) {
-		TCHAR*	psz_ = static_cast<TCHAR*>(psz);
-		size_t	cch = _tcslen(psz_);
+		TCHAR*		psz_ = static_cast<TCHAR*>(psz);
+		std::size_t	cch = _tcslen(psz_);
 		if(cch >= 2 && psz_[cch - 2] == _T('\r') && psz_[cch - 1] == _T('\n'))
 			psz_[cch - 2] = 0;
 		tstring str = psz_;
@@ -324,11 +317,29 @@ inline int CWinApp::LoadString(UINT nID, TCHAR* lpszString, int cchMax) const {
 	return ::LoadString(m_hInstance, nID, lpszString, cchMax);
 }
 
-inline tstring CWinApp::LoadString(UINT nID) const {
+inline tstring CWinApp::LoadString(UINT nID, const CMessageArguments& args /* = CMessageArguments() */) const {
 	AssertValid();
+
 	TCHAR	szString[1024];
 	::LoadString(m_hInstance, nID, szString, 1024);
-	return tstring(szString);
+	if(args.m_listArgs.empty())
+		return tstring(szString);
+
+	void*								psz = 0;
+	TCHAR**								pp = new TCHAR*[args.m_listArgs.size()];
+	std::list<tstring>::const_iterator	it;
+	std::size_t						    i;
+
+	for(it = args.m_listArgs.begin(), i = 0; it != args.m_listArgs.end(); ++it, ++i)
+		pp[i] = const_cast<TCHAR*>(it->c_str());
+	::FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_STRING
+		| FORMAT_MESSAGE_ARGUMENT_ARRAY, szString, 0, 0,
+		reinterpret_cast<TCHAR*>(&psz), 0, reinterpret_cast<va_list*>(pp));
+	delete[] pp;
+
+	tstring	strResult = static_cast<TCHAR*>(psz);
+	::LocalFree(psz);
+	return strResult;
 }
 
 inline bool CWinApp::PreTranslateMessage(MSG* pMsg) {
@@ -391,6 +402,9 @@ inline bool CWinApp::WriteProfileStruct(
 	return toBoolean(::WritePrivateProfileStruct(lpszSectionName, lpszKeyName, lpStruct, uSize, m_szINIPath));
 }
 
+} /* namespace Windows */
+} /* namespace Manah */
+
 #endif /* _WIN_APP_H_ */
 
 /* [EOF] */
\ No newline at end of file
