@@ -229,7 +229,7 @@ public:
 	virtual LRESULT	DefWindowProc(UINT message, WPARAM wParam, LPARAM lParam);
 
 	/* C */
-	bool	AnimateWindow(DWORD dwTime, DWORD dwFlags);
+	bool	AnimateWindow(DWORD dwTime, DWORD dwFlags, bool bCatchError = true);
 	bool	SetLayeredWindowAttributes(COLORREF clrKey, BYTE bAlpha, DWORD dwFlags);
 	bool	UpdateLayeredWindow(HDC hDest, POINT* pptDest,
 				SIZE* pSize, HDC hSrc, POINT* pptSrc, COLORREF clrKey,
@@ -300,16 +300,10 @@ typedef struct _tagMPARAM {
 	LPARAM	lParam;
 } MPARAM;
 
-} /* namespace Controls */
-} /* namespace Windows */
-} /* namespace Manah */
-
 
 // CWindow class implementation
 /////////////////////////////////////////////////////////////////////////////
 
-using Manah::Windows::Controls::CWindow;
-
 inline CWindow::CWindow() : m_hWnd(0), m_OldProc(0),
 		m_lOldUserData(0L), m_hUser32Dll(0),
 		m_pfnAnimateWindow(0), m_pfnSetLayeredWindowAttributes(0), m_pfnUpdateLayeredWindow(0) {
@@ -327,15 +321,23 @@ inline CWindow::~CWindow() {
 		::FreeLibrary(m_hUser32Dll);
 }
 
-inline bool CWindow::AnimateWindow(DWORD dwTime, DWORD dwFlags) {
+inline bool CWindow::AnimateWindow(DWORD dwTime, DWORD dwFlags, bool bCatchError /* = true */) {
 	AssertValidAsWindow();
 	if(m_hUser32Dll == 0)
 		m_hUser32Dll = ::LoadLibrary(_T("User32.dll"));
 	if(m_hUser32Dll != 0 && m_pfnAnimateWindow == 0)
 		m_pfnAnimateWindow =
 			reinterpret_cast<AW>(::GetProcAddress(m_hUser32Dll, "AnimateWindow"));
-	return (m_pfnAnimateWindow != 0) ?
-		toBoolean((*m_pfnAnimateWindow)(m_hWnd, dwTime, dwFlags)) : false;
+	if((m_pfnAnimateWindow != 0) ?
+			toBoolean((*m_pfnAnimateWindow)(m_hWnd, dwTime, dwFlags)) : false)
+		return true;
+	else if(bCatchError) {
+		if(toBoolean(dwFlags & AW_HIDE))
+			return ShowWindow(SW_HIDE);
+		else
+			return ShowWindow(toBoolean(dwFlags & AW_ACTIVATE) ? SW_SHOW : SW_SHOWNA);
+	}
+	return false;
 }
 
 inline UINT CWindow::ArrangeIconicWindows() {
@@ -1471,6 +1473,10 @@ inline void CWindow::OnSysColorChange() {
 inline void CWindow::OnTimer(UINT nIDEvent) {
 }
 
+} /* namespace Controls */
+} /* namespace Windows */
+} /* namespace Manah */
+
 #endif /* _M_WINDOW_H_ */
 
 /* [EOF] */
\ No newline at end of file
